from PyQt5.QtCore import (
    QSize,
    Qt
    )

from PyQt5.QtWidgets import (
    QGridLayout,
    QGroupBox,
    QHBoxLayout,
    QLabel,
    QPushButton,
    QRadioButton,
    QSizePolicy,
    QTableWidget,
    QTableWidgetItem,
    QVBoxLayout,
    QAbstractItemView,
    QLineEdit
    )

class TagSubmitPluginOptionsUI():

    def __init__(self, page):
        self.main_container = QVBoxLayout()

        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)

        # Group box: tag saving
        self.tag_save_groupbox = QGroupBox()
        sizePolicy.setHeightForWidth(self.tag_save_groupbox.sizePolicy().hasHeightForWidth())
        self.tag_save_groupbox.setSizePolicy(sizePolicy)
        self.tag_save_groupbox_layout = QGridLayout(self.tag_save_groupbox)
        # Label: tag saving description
        self.tag_save_description = QLabel(self.tag_save_groupbox)
        sizePolicy.setHeightForWidth(self.tag_save_description.sizePolicy().hasHeightForWidth())
        self.tag_save_description.setSizePolicy(sizePolicy)
        self.tag_save_description.setMinimumSize(QSize(0, 56))
        self.tag_save_description.setWordWrap(True)
        self.tag_save_groupbox_layout.addWidget(self.tag_save_description, 0, 0, 1, 1)
        self.tag_save_groupbox.setTitle("Tag Saving")
        self.tag_save_description.setText(u"<html><head/><body><p>There are two modes to tag saving via this plugin right now: keep all existing saved tags (only adding on tags) or overwrite them. If you are not in a position where replacing your saved tags online is a good idea, it is recommended to keep this option unchanged.</p></body></html>")
        self.main_container.addWidget(self.tag_save_groupbox)
        self.tag_save_groupbox_layout.addWidget(self.tag_save_description, 0, 0, 1, 1)

        self.tags_to_save_groupbox = QGroupBox()
        sizePolicy.setHeightForWidth(self.tags_to_save_groupbox.sizePolicy().hasHeightForWidth())
        self.tags_to_save_groupbox.setSizePolicy(sizePolicy)
        self.tags_to_save_groupbox_layout = QGridLayout(self.tags_to_save_groupbox)
        self.tags_to_save_groupbox.setTitle("Tags to Submit")
        self.tags_to_save_description = QLabel(self.tags_to_save_groupbox)
        sizePolicy.setHeightForWidth(self.tags_to_save_description.sizePolicy().hasHeightForWidth())
        self.tags_to_save_textbox = QLineEdit()
        self.tags_to_save_description.setText("<html><head/><body><p>List the tags that you want to submit to MusicBrainz via the plugin in the text box below. Separate each tag with a semi-colon. (e.g. genre; mood)</p></body></html>")
        self.tags_to_save_groupbox_layout.addWidget(self.tags_to_save_description)
        self.tags_to_save_textbox.setPlaceholderText("Tags you want to submit (separated by semicolons - e.g. genre; mood)")
        self.tags_to_save_groupbox_layout.addWidget(self.tags_to_save_textbox)
        self.main_container.addWidget(self.tags_to_save_groupbox)

        # Radio buttons for tag saving options (on the plugin as "destructive")
        self.keep_radio_button = QRadioButton(self.tag_save_groupbox)
        self.keep_radio_button.setText("Keep existing online saved tags")
        self.tag_save_groupbox_layout.addWidget(self.keep_radio_button, 1, 0, 1, 1)
        self.overwrite_radio_button = QRadioButton(self.tag_save_groupbox)
        self.overwrite_radio_button.setText("Overwrite all online saved tags")
        self.tag_save_groupbox_layout.addWidget(self.overwrite_radio_button, 2, 0, 1, 1)

        # Group box: tag aliases
        self.tag_alias_groupbox = QGroupBox()
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.tag_alias_groupbox.sizePolicy().hasHeightForWidth())
        self.tag_alias_groupbox.setSizePolicy(sizePolicy1)
        self.tag_alias_groupbox.setCheckable(True)
        self.tag_alias_groupbox.setChecked(False)
        self.tag_alias_groupbox_layout = QVBoxLayout(self.tag_alias_groupbox)
        self.tag_alias_groupbox.setTitle("Tag Aliases")
        self.tag_alias_description = QLabel()
        self.tag_alias_description.setText("<html><head/><body><p>There may be cases where you prefer one tag on your files to be saved as another on MusicBrainz (e.g. if your genre tags don't align with MusicBrainz's standard genre tags). In such cases, the plugin can substitute your tags with whichever tags you want when submitting.</p><p>Anything listed here is case-insensitive, as MusicBrainz will process them in lowercase anyway.</p></body></html>")
        self.tag_alias_description.setMinimumSize(QSize(0, 42))
        self.tag_alias_description.setWordWrap(True)
        self.tag_alias_groupbox_layout.addWidget(self.tag_alias_description)

        # Tag alias table
        self.tag_alias_table = QTableWidget()
        self.tag_alias_table.setColumnCount(2)
        __find_column = QTableWidgetItem()
        __find_column.setText("Find...")
        self.tag_alias_table.setHorizontalHeaderItem(0, __find_column)
        __replace_column = QTableWidgetItem()
        __replace_column.setText("Replace...")
        self.tag_alias_table.setHorizontalHeaderItem(1, __replace_column)
        self.tag_alias_table.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.tag_alias_groupbox_layout.addWidget(self.tag_alias_table)

        # Tag alias buttons
        self.table_button_layout = QHBoxLayout()
        self.add_row_button = QPushButton()
        self.delete_row_button = QPushButton()
        self.add_row_button.setText("Add tag alias")
        self.add_row_button.clicked.connect(self.add_row)
        self.delete_row_button.setText("Delete selected tag aliases")
        self.delete_row_button.clicked.connect(self.delete_rows)
        self.table_button_layout.addWidget(self.add_row_button)
        self.table_button_layout.addWidget(self.delete_row_button)
        self.tag_alias_groupbox_layout.addLayout(self.table_button_layout)
        self.main_container.addWidget(self.tag_alias_groupbox)

        page.setLayout(self.main_container)

    def add_row(self, find_entry="", replace_entry=""):
        """
        Adds a row to the table. Accepts input.
        """
        row_pos = self.tag_alias_table.rowCount()
        self.tag_alias_table.insertRow(row_pos)

        find_tableitem = QTableWidgetItem(find_entry)
        replace_tableitem = QTableWidgetItem(replace_entry)

        self.tag_alias_table.setItem(row_pos, 0, find_tableitem)
        self.tag_alias_table.setItem(row_pos, 1, replace_tableitem)

    def delete_rows(self):
        """
        Self-explanatory - removes the selected rows.
        """
        for row in self.tag_alias_table.selectionModel().selectedRows():
            self.tag_alias_table.removeRow(row.row())

    def rows_to_tuple_list(self):
        """
        Converts filled in rows to a list of tuples for the alias list setting.
        """

        tuple_list = []
        row_count = self.tag_alias_table.rowCount()
        for row in range(row_count):
            find_tableitem = self.tag_alias_table.item(row, 0).text()
            replace_tableitem = self.tag_alias_table.item(row, 1).text()
            if find_tableitem and replace_tableitem:
                tuple_list.append((find_tableitem, replace_tableitem))
        return tuple_list



# -*- coding: utf-8 -*-

PLUGIN_NAME = 'Standardise Performers'
PLUGIN_AUTHOR = 'Sophist'
PLUGIN_DESCRIPTION = '''Splits multi-instrument performer tags into single
instruments and combines names so e.g. (from 10cc by 10cc track 1):
<pre>
Performer [acoustic guitar, bass, dobro, electric guitar and tambourine]: Graham Gouldman
Performer [acoustic guitar, electric guitar, grand piano and synthesizer]: Lol Creme
Performer [electric guitar, moog and slide guitar]: Eric Stewart
</pre>
becomes:
<pre>
Performer [acoustic guitar]: Graham Gouldman; Lol Creme
Performer [bass]: Graham Gouldman
Performer [dobro]: Graham Gouldman
Performer [electric guitar]: Eric Stewart; Graham Gouldman; Lol Creme
Performer [grand piano]: Lol Creme
Performer [moog]: Eric Stewart
Performer [slide guitar]: Eric Stewart
Performer [synthesizer]: Lol Creme
Performer [tambourine]: Graham Gouldman
</pre>
Update: This version now sorts the performer tags in order to maintain a consistent value and avoid tags appearing to change even though the base data is equivalent.
'''
PLUGIN_VERSION = '1.0'
PLUGIN_API_VERSIONS = ["2.0"]
PLUGIN_LICENSE = "GPL-2.0"
PLUGIN_LICENSE_URL = "https://www.gnu.org/licenses/gpl-2.0.html"

import re
from picard import log
from picard.metadata import register_track_metadata_processor

standardise_performers_split = re.compile(r", | and ").split


def standardise_performers(album, metadata, *args):
    for key, values in list(metadata.rawitems()):
        if not key.startswith('performer:') \
                and not key.startswith('~performersort:'):
            continue
        mainkey, subkey = key.split(':', 1)
        if not subkey:
            continue
        instruments = standardise_performers_split(subkey)
        if len(instruments) == 1:
            continue
        log.debug("%s: Splitting Performer [%s] into separate performers",
                  PLUGIN_NAME,
                  subkey,
                  )
        prefixes = []
        words = instruments[0].split()
        for word in words[:]:
            if not word in ['guest', 'solo', 'additional', 'minor']:
                break
            prefixes.append(word)
            words.remove(word)
        instruments[0] = " ".join(words)
        prefix = " ".join(prefixes) + " " if prefixes else ""
        for instrument in instruments:
            newkey = '%s:%s%s' % (mainkey, prefix, instrument)
            for value in values:
                metadata.add_unique(newkey, value)
        del metadata[key]

    # Sort performer metdata to avoid changes in processing sequence creating false changes in metadata
    for key, values in list(metadata.rawitems()):
        if key.startswith('performer:') or key.startswith('~performersort:'):
            metadata[key] = sorted(values)


from picard.plugin import PluginPriority
register_track_metadata_processor(standardise_performers,
                                  priority=PluginPriority.HIGH)



#!/usr/bin/env python3

import glob
import os

from PyQt5 import uic


os.chdir(os.path.dirname(__file__))
plugin_dir = os.path.relpath('plugins')


def compile_ui(uifile, pyfile):
    if newer(uifile, pyfile):
        print("compiling %s -> %s" % (uifile, pyfile))
        with open(pyfile, "w") as out:
            uic.compileUi(uifile, out)
    else:
        print("skipping %s -> %s: up to date" % (uifile, pyfile))


def newer(file1, file2):
    """Returns True, if file1 has been modified after file2
    """
    if not os.path.exists(file2):
        return True
    return os.path.getmtime(file1) > os.path.getmtime(file2)


if __name__ == '__main__':
    for uifile in glob.glob(os.path.join(plugin_dir, '*/*.ui')):
        pyfile = os.path.splitext(os.path.basename(uifile))[0] + '.py'
        pyfile = os.path.join(os.path.dirname(uifile), pyfile)
        compile_ui(uifile, pyfile)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'plugins/replaygain2/ui_options_replaygain2.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ReplayGain2OptionsPage(object):
    def setupUi(self, ReplayGain2OptionsPage):
        ReplayGain2OptionsPage.setObjectName("ReplayGain2OptionsPage")
        ReplayGain2OptionsPage.resize(477, 577)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ReplayGain2OptionsPage.sizePolicy().hasHeightForWidth())
        ReplayGain2OptionsPage.setSizePolicy(sizePolicy)
        self.vboxlayout = QtWidgets.QVBoxLayout(ReplayGain2OptionsPage)
        self.vboxlayout.setContentsMargins(9, 9, 9, 9)
        self.vboxlayout.setSpacing(6)
        self.vboxlayout.setObjectName("vboxlayout")
        self.replay_gain = QtWidgets.QGroupBox(ReplayGain2OptionsPage)
        self.replay_gain.setObjectName("replay_gain")
        self.vboxlayout1 = QtWidgets.QVBoxLayout(self.replay_gain)
        self.vboxlayout1.setContentsMargins(9, 9, 9, 9)
        self.vboxlayout1.setSpacing(2)
        self.vboxlayout1.setObjectName("vboxlayout1")
        self.label = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.vboxlayout1.addWidget(self.label)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.rsgain_command = QtWidgets.QLineEdit(self.replay_gain)
        self.rsgain_command.setObjectName("rsgain_command")
        self.horizontalLayout.addWidget(self.rsgain_command)
        self.rsgain_command_browse = QtWidgets.QToolButton(self.replay_gain)
        self.rsgain_command_browse.setObjectName("rsgain_command_browse")
        self.horizontalLayout.addWidget(self.rsgain_command_browse)
        self.vboxlayout1.addLayout(self.horizontalLayout)
        self.label_6 = QtWidgets.QLabel(self.replay_gain)
        self.label_6.setOpenExternalLinks(True)
        self.label_6.setTextInteractionFlags(QtCore.Qt.TextBrowserInteraction)
        self.label_6.setObjectName("label_6")
        self.vboxlayout1.addWidget(self.label_6)
        self.album_tags = QtWidgets.QCheckBox(self.replay_gain)
        self.album_tags.setChecked(True)
        self.album_tags.setObjectName("album_tags")
        self.vboxlayout1.addWidget(self.album_tags)
        self.true_peak = QtWidgets.QCheckBox(self.replay_gain)
        self.true_peak.setObjectName("true_peak")
        self.vboxlayout1.addWidget(self.true_peak)
        self.reference_loudness = QtWidgets.QCheckBox(self.replay_gain)
        self.reference_loudness.setObjectName("reference_loudness")
        self.vboxlayout1.addWidget(self.reference_loudness)
        self.label_5 = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.vboxlayout1.addWidget(self.label_5)
        self.target_loudness = QtWidgets.QSpinBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.target_loudness.sizePolicy().hasHeightForWidth())
        self.target_loudness.setSizePolicy(sizePolicy)
        self.target_loudness.setMinimum(-30)
        self.target_loudness.setMaximum(-5)
        self.target_loudness.setProperty("value", -18)
        self.target_loudness.setObjectName("target_loudness")
        self.vboxlayout1.addWidget(self.target_loudness)
        self.label_2 = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.vboxlayout1.addWidget(self.label_2)
        self.clip_mode = QtWidgets.QComboBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.clip_mode.sizePolicy().hasHeightForWidth())
        self.clip_mode.setSizePolicy(sizePolicy)
        self.clip_mode.setMaxCount(3)
        self.clip_mode.setObjectName("clip_mode")
        self.vboxlayout1.addWidget(self.clip_mode)
        self.label_3 = QtWidgets.QLabel(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.vboxlayout1.addWidget(self.label_3)
        self.max_peak = QtWidgets.QSpinBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_peak.sizePolicy().hasHeightForWidth())
        self.max_peak.setSizePolicy(sizePolicy)
        self.max_peak.setMinimum(-5)
        self.max_peak.setMaximum(0)
        self.max_peak.setObjectName("max_peak")
        self.vboxlayout1.addWidget(self.max_peak)
        self.label_4 = QtWidgets.QLabel(self.replay_gain)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.vboxlayout1.addWidget(self.label_4)
        self.opus_mode = QtWidgets.QComboBox(self.replay_gain)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.opus_mode.sizePolicy().hasHeightForWidth())
        self.opus_mode.setSizePolicy(sizePolicy)
        self.opus_mode.setMaxCount(3)
        self.opus_mode.setObjectName("opus_mode")
        self.vboxlayout1.addWidget(self.opus_mode)
        self.opus_m23 = QtWidgets.QCheckBox(self.replay_gain)
        self.opus_m23.setObjectName("opus_m23")
        self.vboxlayout1.addWidget(self.opus_m23)
        self.vboxlayout.addWidget(self.replay_gain)
        spacerItem = QtWidgets.QSpacerItem(263, 21, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.vboxlayout.addItem(spacerItem)

        self.retranslateUi(ReplayGain2OptionsPage)
        self.opus_mode.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(ReplayGain2OptionsPage)

    def retranslateUi(self, ReplayGain2OptionsPage):
        _translate = QtCore.QCoreApplication.translate
        self.replay_gain.setTitle(_translate("ReplayGain2OptionsPage", "ReplayGain 2.0 Settings"))
        self.label.setText(_translate("ReplayGain2OptionsPage", "Path to rsgain:"))
        self.rsgain_command.setText(_translate("ReplayGain2OptionsPage", "rsgain"))
        self.rsgain_command_browse.setText(_translate("ReplayGain2OptionsPage", "Browse..."))
        self.label_6.setText(_translate("ReplayGain2OptionsPage", "<a href=\"https://github.com/complexlogic/rsgain\">Download rsgain</a>"))
        self.album_tags.setText(_translate("ReplayGain2OptionsPage", "Calculate album gain/peak"))
        self.true_peak.setWhatsThis(_translate("ReplayGain2OptionsPage", "Use more accurate true peak calculations (trade for performance)"))
        self.true_peak.setText(_translate("ReplayGain2OptionsPage", "Use true peak"))
        self.reference_loudness.setText(_translate("ReplayGain2OptionsPage", "Write reference loudness tags"))
        self.label_5.setText(_translate("ReplayGain2OptionsPage", "Target Loudness (LUFS)"))
        self.label_2.setText(_translate("ReplayGain2OptionsPage", "Clipping Protection"))
        self.label_3.setText(_translate("ReplayGain2OptionsPage", "Max Peak (dB)"))
        self.label_4.setText(_translate("ReplayGain2OptionsPage", "Opus Files"))
        self.opus_m23.setText(_translate("ReplayGain2OptionsPage", "Always reference Opus R128_*_GAIN tags to -23 LUFS"))



# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
# Copyright (C) 2007 Oliver Charles
# Copyright (C) 2007-2011, 2019, 2021 Philipp Wolfer
# Copyright (C) 2007, 2010, 2011 Lukáš Lalinský
# Copyright (C) 2011 Michael Wiencek
# Copyright (C) 2011-2012 Wieland Hoffmann
# Copyright (C) 2013-2016 Laurent Monin
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

PLUGIN_NAME = 'Amazon cover art'
PLUGIN_AUTHOR = 'MusicBrainz Picard developers'
PLUGIN_DESCRIPTION = 'Use cover art from Amazon.'
PLUGIN_VERSION = "1.1"
PLUGIN_API_VERSIONS = ["2.2", "2.3", "2.4", "2.5", "2.6", "2.7"]
PLUGIN_LICENSE = "GPL-2.0-or-later"
PLUGIN_LICENSE_URL = "https://www.gnu.org/licenses/gpl-2.0.html"

from picard import log
from picard.coverart.image import CoverArtImage
from picard.coverart.providers import (
    CoverArtProvider,
    register_cover_art_provider,
)
from picard.util import parse_amazon_url

# amazon image file names are unique on all servers and constructed like
# <ASIN>.<ServerNumber>.[SML]ZZZZZZZ.jpg
# A release sold on amazon.de has always <ServerNumber> = 03, for example.
# Releases not sold on amazon.com, don't have a "01"-version of the image,
# so we need to make sure we grab an existing image.
AMAZON_SERVER = {
    "amazon.com": {
        "server": "ec1.images-amazon.com",
        "id": "01",
    },
    "amazon.jp": {
        "server": "ec1.images-amazon.com",
        "id": "09",
    },
    "amazon.co.jp": {
        "server": "ec1.images-amazon.com",
        "id": "09",
    },
    "amazon.co.uk": {
        "server": "ec1.images-amazon.com",
        "id": "02",
    },
    "amazon.de": {
        "server": "ec2.images-amazon.com",
        "id": "03",
    },
    "amazon.ca": {
        "server": "ec1.images-amazon.com",
        "id": "01",  # .com and .ca are identical
    },
    "amazon.fr": {
        "server": "ec1.images-amazon.com",
        "id": "08"
    },
}

AMAZON_IMAGE_PATH = '/images/P/%(asin)s.%(serverid)s.%(size)s.jpg'

# First item in the list will be tried first
AMAZON_SIZES = (
    # huge size option is only available for items
    # that have a ZOOMing picture on its amazon web page
    # and it doesn't work for all of the domain names
    # '_SCRM_',        # huge size
    'LZZZZZZZ',      # large size, option format 1
    # '_SCLZZZZZZZ_',  # large size, option format 3
    'MZZZZZZZ',      # default image size, format 1
    # '_SCMZZZZZZZ_',  # medium size, option format 3
    # 'TZZZZZZZ',      # medium image size, option format 1
    # '_SCTZZZZZZZ_',  # small size, option format 3
    # 'THUMBZZZ',      # small size, option format 1
)


class CoverArtProviderAmazon(CoverArtProvider):

    """Use Amazon ASIN Musicbrainz relationships to get cover art"""

    NAME = "Amazon"
    TITLE = N_('Amazon')

    def __init__(self, coverart):
        super().__init__(coverart)
        self._has_url_relation = False

    def enabled(self):
        return (super().enabled()
                and not self.coverart.front_image_found)

    def queue_images(self):
        self.match_url_relations(('amazon asin', 'has_Amazon_ASIN'),
                                 self._queue_from_asin_relation)
        # No URL relationships loaded, try by ASIN
        if not self._has_url_relation:
            self._queue_from_asin()
        return CoverArtProvider.FINISHED

    def _queue_from_asin_relation(self, url):
        """Queue cover art images from Amazon"""
        amz = parse_amazon_url(url)
        if amz is None:
            return
        log.debug("Found ASIN relation : %s %s", amz['host'], amz['asin'])
        self._has_url_relation = True
        if amz['host'] in AMAZON_SERVER:
            server_info = AMAZON_SERVER[amz['host']]
        else:
            server_info = AMAZON_SERVER['amazon.com']
        self._queue_asin(server_info, amz['asin'])

    def _queue_from_asin(self):
        asin = self.release.get('asin')
        if asin:
            log.debug("Found ASIN : %s", asin)
            for server_info in AMAZON_SERVER.values():
               self._queue_asin(server_info, asin)

    def _queue_asin(self, server_info, asin):
        host = server_info['server']
        for size in AMAZON_SIZES:
            path = AMAZON_IMAGE_PATH % {
                'asin': asin,
                'serverid': server_info['id'],
                'size': size
            }
            self.queue_put(CoverArtImage("http://%s%s" % (host, path)))


register_cover_art_provider(CoverArtProviderAmazon)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Bob Swift (rdswift)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

PLUGIN_NAME = 'Genre Mapper'
PLUGIN_AUTHOR = 'Bob Swift'
PLUGIN_DESCRIPTION = '''
This plugin provides the ability to standardize genres in the "genre"
tag by matching the genres as found to a standard genre as defined in
the genre replacement mapping configuration option. Once installed a
settings page will be added to Picard's options, which is where the
plugin is configured.
<br /><br />
Please see the <a href="https://github.com/rdswift/picard-plugins/blob/2.0_RDS_Plugins/plugins/genre_mapper/docs/README.md">user guide</a> on GitHub for more information.
'''
PLUGIN_VERSION = '0.6'
PLUGIN_API_VERSIONS = ['2.0', '2.1', '2.2', '2.3', '2.6', '2.7', '2.8', '2.9', '2.10', '2.11']
PLUGIN_LICENSE = "GPL-2.0"
PLUGIN_LICENSE_URL = "https://www.gnu.org/licenses/gpl-2.0.txt"

import re

from picard import (
    config,
    log,
)
from picard.metadata import (
    MULTI_VALUED_JOINER,
    register_track_metadata_processor,
)
from picard.plugin import PluginPriority
from picard.plugins.genre_mapper.ui_options_genre_mapper import (
    Ui_GenreMapperOptionsPage,
)

from picard.ui.options import (
    OptionsPage,
    register_options_page,
)


pairs_split = re.compile(r"\r\n|\n\r|\n").split

OPT_GENRE_SEPARATOR = 'join_genres'
OPT_MATCH_ENABLED = 'genre_mapper_enabled'
OPT_MATCH_PAIRS = 'genre_mapper_replacement_pairs'
OPT_MATCH_FIRST = 'genre_mapper_apply_first_match_only'
OPT_MATCH_REGEX = 'genre_mapper_use_regex'


class GenreMappingPairs():
    pairs = []

    @classmethod
    def refresh(cls):
        log.debug("%s: Refreshing the genre replacement maps processing pairs using '%s' translation.",
            PLUGIN_NAME, 'RegEx' if config.Option.exists("setting", OPT_MATCH_REGEX) and config.setting[OPT_MATCH_REGEX] else 'Simple',)
        if not config.Option.exists("setting", OPT_MATCH_PAIRS):
            log.warning("%s: Unable to read the '%s' setting.", PLUGIN_NAME, OPT_MATCH_PAIRS,)
            return

        def _make_re(map_string):
            # Replace period with temporary placeholder character (newline)
            re_string = str(map_string).strip().replace('.', '\n')
            # Convert wildcard characters to regular expression equivalents
            re_string = re_string.replace('*', '.*').replace('?', '.')
            # Escape carat and dollar sign for regular expression
            re_string = re_string.replace('^', '\\^').replace('$', '\\$')
            # Replace temporary placeholder characters with escaped periods
            re_string = '^' + re_string.replace('\n', '\\.') + '$'
            # Return regular expression with carat and dollar sign to force match condition on full string
            return re_string

        cls.pairs = []
        for pair in pairs_split(config.setting[OPT_MATCH_PAIRS]):
            if "=" not in pair:
                continue
            original, replacement = pair.split('=', 1)
            original = original.strip()
            if not original:
                continue
            replacement = replacement.strip()
            cls.pairs.append((original if config.setting[OPT_MATCH_REGEX] else _make_re(original), replacement))
            log.debug('%s: Add genre mapping pair: "%s" = "%s"', PLUGIN_NAME, original, replacement,)
        if not cls.pairs:
            log.debug("%s: No genre replacement maps defined.", PLUGIN_NAME,)


class GenreMapperOptionsPage(OptionsPage):

    NAME = "genre_mapper"
    TITLE = "Genre Mapper"
    PARENT = "plugins"

    options = [
        config.TextOption("setting", OPT_MATCH_PAIRS, ''),
        config.BoolOption("setting", OPT_MATCH_FIRST, False),
        config.BoolOption("setting", OPT_MATCH_ENABLED, False),
        config.BoolOption("setting", OPT_MATCH_REGEX, False),
    ]

    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_GenreMapperOptionsPage()
        self.ui.setupUi(self)

    def load(self):
        # Enable external link
        self.ui.format_description.setOpenExternalLinks(True)

        self.ui.genre_mapper_replacement_pairs.setPlainText(config.setting[OPT_MATCH_PAIRS])
        self.ui.genre_mapper_first_match_only.setChecked(config.setting[OPT_MATCH_FIRST])
        self.ui.cb_enable_genre_mapping.setChecked(config.setting[OPT_MATCH_ENABLED])
        self.ui.cb_use_regex.setChecked(config.setting[OPT_MATCH_REGEX])

        self.ui.cb_enable_genre_mapping.stateChanged.connect(self._set_enabled_state)
        self._set_enabled_state()

    def save(self):
        config.setting[OPT_MATCH_PAIRS] = self.ui.genre_mapper_replacement_pairs.toPlainText()
        config.setting[OPT_MATCH_FIRST] = self.ui.genre_mapper_first_match_only.isChecked()
        config.setting[OPT_MATCH_ENABLED] = self.ui.cb_enable_genre_mapping.isChecked()
        config.setting[OPT_MATCH_REGEX] = self.ui.cb_use_regex.isChecked()

        GenreMappingPairs.refresh()

    def _set_enabled_state(self, *args):
        self.ui.gm_replacement_pairs.setEnabled(self.ui.cb_enable_genre_mapping.isChecked())


def track_genre_mapper(album, metadata, *args):
    if not config.setting[OPT_MATCH_ENABLED]:
        return
    if 'genre' not in metadata or not metadata['genre']:
        log.debug('%s: No genres found for: "%s"', PLUGIN_NAME, metadata['title'],)
        return
    genre_joiner = config.setting[OPT_GENRE_SEPARATOR] if config.setting[OPT_GENRE_SEPARATOR] else MULTI_VALUED_JOINER
    genres = set()
    metadata_genres = str(metadata['genre']).split(genre_joiner)
    for genre in metadata_genres:
        for (original, replacement) in GenreMappingPairs.pairs:
            try:
                if genre and re.search(original, genre, re.IGNORECASE):
                    genre = replacement
                    if config.setting[OPT_MATCH_FIRST]:
                        break
            except re.error:
                log.error('%s: Invalid regular expression ignored: "%s"', PLUGIN_NAME, original,)
        if genre:
            genres.add(genre.title())
    genres = sorted(genres)
    log.debug('%s: Genres updated from %s to %s', PLUGIN_NAME, metadata_genres, genres,)
    metadata['genre'] = genres


# Register the plugin to run at a LOW priority.
register_track_metadata_processor(track_genre_mapper, priority=PluginPriority.LOW)
register_options_page(GenreMapperOptionsPage)

GenreMappingPairs.refresh()



# -*- coding: utf-8 -*-
"""Additional Artists Details
"""
# Copyright (C) 2023-2024 Bob Swift (rdswift)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

# pylint: disable=line-too-long
# pylint: disable=import-error
# pylint: disable=too-many-arguments
# pylint: disable=too-many-locals

from collections import namedtuple
from functools import partial

from picard import (
    config,
    log,
)
from picard.album import register_album_post_removal_processor
from picard.metadata import (
    register_album_metadata_processor,
    register_track_metadata_processor,
)
from picard.plugin import PluginPriority
from picard.plugins.additional_artists_details.ui_options_additional_artists_details import (
    Ui_AdditionalArtistsDetailsOptionsPage,
)
from picard.webservice.api_helpers import MBAPIHelper

from picard.ui.options import (
    OptionsPage,
    register_options_page,
)


PLUGIN_NAME = 'Additional Artists Details'
PLUGIN_AUTHOR = 'Bob Swift (rdswift)'
PLUGIN_DESCRIPTION = '''
This plugin provides specialized album and track variables with artist details for use in tagging and naming scripts.  Note that this creates
additional calls to the MusicBrainz API for the artist and area information, and this will slow down processing.  This will be particularly
noticable when there are many different album or track artists, such as on a [Various Artists] release.  There is an option to disable track
artist processing, which can significantly increase the processing speed if you are only interested in album artist details.
<br /><br />
Please see the <a href="https://github.com/rdswift/picard-plugins/blob/2.0_RDS_Plugins/plugins/additional_artists_details/docs/README.md">user
guide</a> on GitHub for more information.
'''
PLUGIN_VERSION = '0.4'
PLUGIN_API_VERSIONS = ['2.0', '2.1', '2.2', '2.7', '2.8', '2.11']
PLUGIN_LICENSE = 'GPL-2.0-or-later'
PLUGIN_LICENSE_URL = 'https://www.gnu.org/licenses/gpl-2.0.html'

PLUGIN_USER_GUIDE_URL = 'https://github.com/rdswift/picard-plugins/blob/2.0_RDS_Plugins/plugins/additional_artists_details/docs/README.md'

# Named tuples for code clarity
Area = namedtuple('Area', ['parent', 'name', 'country', 'type', 'type_text'])
MetadataPair = namedtuple('MetadataPair', ['artists', 'target'])

# MusicBrainz ID codes for relationship types
RELATIONSHIP_TYPE_PART_OF = 'de7cc874-8b1b-3a05-8272-f3834c968fb7'

# MusicBrainz ID codes for area types
AREA_TYPE_COUNTRY = '06dd0ae4-8c74-30bb-b43d-95dcedf961de'
AREA_TYPE_COUNTY = 'bcecec27-8bdb-3e00-8254-d948dda502fa'
AREA_TYPE_MUNICIPALITY = '17246454-5ac4-36a1-b81a-4753eb2dab20'

# Area types to exclude from the location string
EXCLUDE_AREA_TYPES = {AREA_TYPE_MUNICIPALITY, AREA_TYPE_COUNTY}

# Standard text for arguments
ALBUM_ARTISTS = 'album_artists'
ARTIST = 'artist'
ARTIST_REQUESTS = 'artist_requests'
AREA = 'area'
AREA_REQUESTS = 'area_requests'
ISO_CODES_1 = 'iso-3166-1-codes'
ISO_CODES_2 = 'iso-3166-2-codes'
OPT_AREA_DETAILS = 'aad_area_details'
OPT_PROCESS_TRACKS = 'aad_process_tracks'
TRACKS = 'tracks'


def log_helper(text, *args):
    """Logging helper to prepend the plugin name to the text.

    Args:
        text (str): Text to log.
        args (list): List of text replacement arguments.

    Returns:
        list: updated text and replacement arguments.
    """
    retval = ["%s: " + text, PLUGIN_NAME]
    retval.extend(args)
    return retval


class CustomHelper(MBAPIHelper):
    """Custom MusicBrainz API helper to retrieve artist and area information.
    """

    def get_artist_by_id(self, _id, handler, inc=None, priority=False, important=False,
                         mblogin=False, refresh=False):
        """Get information for the specified artist MBID.

        Args:
            _id (str): Artist MBID to retrieve.
            handler (object): Callback used to process the returned information.
            inc (list, optional): List of includes to add to the API call. Defaults to None.
            priority (bool, optional): Process the request at a high priority. Defaults to False.
            important (bool, optional): Identify the request as important. Defaults to False.
            mblogin (bool, optional): Request requires logging into MusicBrainz. Defaults to False.
            refresh (bool, optional): Request triggers a refresh. Defaults to False.

        Returns:
            RequestTask: Requested task object.
        """
        return self._get_by_id(ARTIST, _id, handler, inc, priority=priority, important=important, mblogin=mblogin, refresh=refresh)

    def get_area_by_id(self, _id, handler, inc=None, priority=False, important=False, mblogin=False, refresh=False):
        """Get information for the specified area MBID.

        Args:
            _id (str): Area MBID to retrieve.
            handler (object): Callback used to process the returned information.
            inc (list, optional): List of includes to add to the API call. Defaults to None.
            priority (bool, optional): Process the request at a high priority. Defaults to False.
            important (bool, optional): Identify the request as important. Defaults to False.
            mblogin (bool, optional): Request requires logging into MusicBrainz. Defaults to False.
            refresh (bool, optional): Request triggers a refresh. Defaults to False.

        Returns:
            RequestTask: Requested task object.
        """
        if inc is None:
            inc = ['area-rels']
        return self._get_by_id(AREA, _id, handler, inc, priority=priority, important=important, mblogin=mblogin, refresh=refresh)


class ArtistDetailsPlugin:
    """Plugin to retrieve artist details, including area and country information.
    """
    result_cache = {
        ARTIST: {},
        ARTIST_REQUESTS: set(),
        AREA: {},
        AREA_REQUESTS: set(),
    }
    album_processing_count = {}
    albums = {}

    def _make_empty_target(self, album_id):
        """Create an empty album target node if it doesn't exist.

        Args:
            album_id (str): MBID of the album.
        """
        if album_id not in self.albums:
            self.albums[album_id] = {ALBUM_ARTISTS: set(), TRACKS: []}

    def _add_target(self, album_id, artists, target_metadata):
        """Add a metadata target to update for an album.

        Args:
            album_id (str): MBID of the album.
            artists (set): Set of artists to include.
            target_metadata (Metadata): Target metadata to update.
        """
        self._make_empty_target(album_id)
        self.albums[album_id][TRACKS].append(MetadataPair(artists, target_metadata))

    def _remove_album(self, album_id):
        """Removes an album from the metadata processing dictionary.

        Args:
            album_id (str): MBID of the album to remove.
        """
        log.debug(*log_helper("Removing album '%s'", album_id))
        self.albums.pop(album_id, None)
        self.album_processing_count.pop(album_id, None)

    def _album_add_request(self, album):
        """Increment the number of pending requests for an album.

        Args:
            album (Album): The Album object to use for the processing.
        """
        if album.id not in self.album_processing_count:
            self.album_processing_count[album.id] = 0
        self.album_processing_count[album.id] += 1
        album._requests += 1

    def _album_remove_request(self, album):
        """Decrement the number of pending requests for an album.

        Args:
            album (Album): The Album object to use for the processing.
        """
        if album.id not in self.album_processing_count:
            self.album_processing_count[album.id] = 1
        self.album_processing_count[album.id] -= 1
        album._requests -= 1
        album._finalize_loading(None)   # pylint: disable=protected-access

    def remove_album(self, album):
        """Remove the album from the albums processing dictionary.

        Args:
            album (Album): The album object to remove.
        """
        self._remove_album(album.id)

    def make_album_vars(self, album, album_metadata, _release_metadata):
        """Process album artists.

        Args:
            album (Album): The Album object to use for the processing.
            album_metadata (Metadata): Metadata object for the album.
            _release_metadata (dict): Dictionary of release data from MusicBrainz api.
        """
        artists = set(artist.id for artist in album.get_album_artists())
        self._make_empty_target(album.id)
        self.albums[album.id][ALBUM_ARTISTS] = artists
        if not config.setting[OPT_PROCESS_TRACKS]:
            log.info(*log_helper("Track artist processing is disabled."))
        self._artist_processing(artists, album, album_metadata, 'Album')

    def make_track_vars(self, album, album_metadata, track_metadata, _release_metadata):
        """Process track artists.

        Args:
            album (Album): The Album object to use for the processing.
            album_metadata (Metadata): Metadata object for the album.
            track_metadata (dict): Dictionary of track data from MusicBrainz api.
            _release_metadata (dict): Dictionary of release data from MusicBrainz api.
        """
        artists = set()
        source_type = 'track'
        # Test for valid metadata node.
        # The 'artist-credit' key should always be there.
        # This check is to avoid a runtime error if it doesn't exist for some reason.
        if config.setting[OPT_PROCESS_TRACKS]:
            if 'artist-credit' in track_metadata:
                for artist_credit in track_metadata['artist-credit']:
                    if 'artist' in artist_credit:
                        if 'id' in artist_credit['artist']:
                            artists.add(artist_credit['artist']['id'])
                    else:
                        # No 'artist' specified.  Log as an error.
                        self._metadata_error(album.id, 'artist-credit.artist', source_type)
            else:
                # No valid metadata found.  Log as error.
                self._metadata_error(album.id, 'artist-credit', source_type)
        self._artist_processing(artists, album, album_metadata, 'Track')

    def _artist_processing(self, artists, album, destination_metadata, source_type):
        """Retrieves the information for each artist not already processed.

        Args:
            artists (set): Set of artist MBIDs to process.
            album (Album): Album object to use for the processing.
            destination_metadata (Metadata): Metadata object to update with the new variables.
            source_type (str): Source type (album or track) for logging messages.
        """
        for temp_id in artists:
            if temp_id not in self.result_cache[ARTIST_REQUESTS]:
                self.result_cache[ARTIST_REQUESTS].add(temp_id)
                log.debug(*log_helper('Retrieving artist ID %s information from MusicBrainz.', temp_id))
                self._get_artist_info(temp_id, album)
            else:
                log.debug(*log_helper('%s artist ID %s information available from cache.', source_type, temp_id))
        self._add_target(album.id, artists, destination_metadata)
        self._save_artist_metadata(album.id)

    def _save_artist_metadata(self, album_id):
        """Saves the new artist details variables to the metadata targets for the specified album.

        Args:
            album_id (str): MBID of the album to process.
        """
        if album_id in self.album_processing_count and self.album_processing_count[album_id]:
            return
        if album_id not in self.albums or not self.albums[album_id][TRACKS]:
            log.error(*log_helper("No metadata targets found for album '%s'", album_id))
            return
        for item in self.albums[album_id][TRACKS]:
            # Add album artists to track so they are available in the metadata
            artists = self.albums[album_id][ALBUM_ARTISTS].copy().union(item.artists)
            destination_metadata = item.target
            for artist in artists:
                if artist in self.result_cache[ARTIST]:
                    self._set_artist_metadata(destination_metadata, artist, self.result_cache[ARTIST][artist])

    def _set_artist_metadata(self, destination_metadata, artist_id, artist_info):
        """Adds the artist information to the destination metadata.

        Args:
            destination_metadata (Metadata): Metadata object to update with new variables.
            artist_id (str): MBID of the artist to update.
            artist_info (dict): Dictionary of information for the artist.
        """
        def _set_item(key, value):
            destination_metadata[f"~artist_{artist_id}_{key.replace('-', '_')}"] = value

        for item in artist_info.keys():
            if item in {'area', 'begin-area', 'end-area'}:
                country, location = self._drill_area(artist_info[item])
                if country:
                    _set_item(item.replace('area', 'country'), country)
                if location:
                    _set_item(item.replace('area', 'location'), location)
            else:
                _set_item(item, artist_info[item])

    def _get_artist_info(self, artist_id, album):
        """Gets the artist information from the MusicBrainz website.

        Args:
            artist_id (str): MBID of the artist to retrieve.
            album (Album): The Album object to use for the processing.
        """
        self._album_add_request(album)
        helper = CustomHelper(album.tagger.webservice)
        handler = partial(
            self._artist_submission_handler,
            artist=artist_id,
            album=album,
        )
        return helper.get_artist_by_id(artist_id, handler)

    def _artist_submission_handler(self, document, _reply, error, artist=None, album=None):
        """Handles the response from the webservice requests for artist information.
        """
        try:
            if error:
                log.error(*log_helper("Artist '%s' information retrieval error.", artist))
                return
            artist_info = {}
            for item in ['type', 'gender', 'name', 'sort-name', 'disambiguation']:
                if item in document and document[item]:
                    artist_info[item] = document[item]
            if 'life-span' in document:
                for item in ['begin', 'end']:
                    if item in document['life-span'] and document['life-span'][item]:
                        artist_info[item] = document['life-span'][item]
            for item in ['area', 'begin-area', 'end-area']:
                if item in document and document[item] and 'id' in document[item] and document[item]['id']:
                    area_id = document[item]['id']
                    artist_info[item] = area_id
                    if area_id not in self.result_cache[AREA_REQUESTS]:
                        self._get_area_info(area_id, album)
            self.result_cache[ARTIST][artist] = artist_info
        finally:
            self._album_remove_request(album)
            self._save_artist_metadata(album.id)

    def _get_area_info(self, area_id, album):
        """Gets the area information from the MusicBrainz website.

        Args:
            area_id (str): MBID of the area to retrieve.
            album (Album): The Album object to use for the processing.
        """
        self.result_cache[AREA_REQUESTS].add(area_id)
        self._album_add_request(album)
        log.debug(*log_helper('Retrieving area ID %s from MusicBrainz.', area_id))
        helper = CustomHelper(album.tagger.webservice)
        handler = partial(
            self._area_submission_handler,
            area=area_id,
            album=album,
        )
        return helper.get_area_by_id(area_id, handler)

    def _area_submission_handler(self, document, _reply, error, area=None, album=None):
        """Handles the response from the webservice requests for area information.
        """
        try:
            if error:
                log.error(*log_helper("Area '%s' information retrieval error.", area))
                return
            (_id, name, country, _type, type_text) = self._parse_area(document)
            if _type == AREA_TYPE_COUNTRY and _id not in self.result_cache[AREA]:
                self._area_logger(_id, f"{name} ({country})", type_text)
                self.result_cache[AREA][_id] = Area('', name, country, _type, type_text)
            if 'relations' in document:
                for rel in document['relations']:
                    self._parse_area_relation(_id, rel, album, name, _type, type_text)
        finally:
            self._album_remove_request(album)
            self._save_artist_metadata(album.id)

    @staticmethod
    def _area_logger(area_id, area_name, area_type):
        """Adds a log entry for the area retrieved.

        Args:
            area_id (str): MBID of the area added.
            area_name (str): Name of the area added.
            area_type (str): Type of area added.
        """
        log.debug(*log_helper("Adding area: %s => %s of type '%s'", area_id, area_name, area_type))

    def _parse_area_relation(self, area_id, area_relation, album, area_name, area_type, area_type_text):
        """Parse an area relation to extract the area information.

        Args:
            area_id (str): MBID of the area providing the relationship.
            area_relation (dict): Dictionary of the area relationship.
            album (Album): The Album object to use for the processing.
            area_name (str): Name of the area providing the relationship.
            area_type (str): MBID of the type of area providing the relationship.
            area_type_text (str): Text description of the area providing the relationship.
        """
        if 'type-id' not in area_relation or 'area' not in area_relation or area_relation['type-id'] != RELATIONSHIP_TYPE_PART_OF:
            return
        (_id, name, country, _type, type_text) = self._parse_area(area_relation['area'])
        if not _id:
            return

        if 'direction' in area_relation and area_relation['direction'] == 'backward':
            if area_id not in self.result_cache[AREA]:
                self._area_logger(area_id, area_name, area_type_text)
                self.result_cache[AREA][area_id] = Area(_id, area_name, '', area_type, type_text)
                self.result_cache[AREA_REQUESTS].add(area_id)
            if _type == AREA_TYPE_COUNTRY:
                if _id not in self.result_cache[AREA]:
                    self._area_logger(_id, f"{name} ({country})", type_text)
                    self.result_cache[AREA][_id] = Area('', name, country, _type, type_text)
                    self.result_cache[AREA_REQUESTS].add(_id)
            else:
                if _id not in self.result_cache[AREA] and _id not in self.result_cache[AREA_REQUESTS]:
                    self._get_area_info(_id, album)
        else:
            self._area_logger(_id, name, type_text)
            self.result_cache[AREA_REQUESTS].add(_id)
            self.result_cache[AREA][_id] = Area(area_id, name, '', _type, type_text)

    @staticmethod
    def _parse_area(area_info):
        """Parse a dictionary of area information to return selected elements.

        Args:
            area_info (dict): Area information to parse.

        Returns:
            tuple: Selected information for the area (id, name, country code, type code, type text).
        """
        if 'id' not in area_info:
            return ('', '', '', '', '')
        area_id = area_info['id']
        area_name = area_info['name'] if 'name' in area_info else 'Unknown Name'
        area_type = area_info['type-id'] if 'type-id' in area_info else ''
        area_type_text = area_info['type'] if 'type' in area_info else 'Unknown Area Type'
        country = ''
        if area_type == AREA_TYPE_COUNTRY:
            if ISO_CODES_1 in area_info and area_info[ISO_CODES_1]:
                country = area_info[ISO_CODES_1][0]
            elif ISO_CODES_2 in area_info and area_info[ISO_CODES_2]:
                country = area_info[ISO_CODES_2][0][:2]
        return (area_id, area_name, country, area_type, area_type_text)

    @staticmethod
    def _metadata_error(album_id, metadata_element, metadata_group):
        """Logs metadata-related errors.

        Args:
            album_id (str): MBID of the album being processed.
            metadata_element (str): Metadata element initiating the error.
            metadata_group (str): Metadata group initiating the error.
        """
        log.error(*log_helper("Album '%s' missing '%s' in %s metadata.", album_id, metadata_element, metadata_group))

    def _drill_area(self, area_id):
        """Drills up from the specified area to determine the two-character
        country code and the full location description for the area.

        Args:
            area_id (str): MBID of the area to process.

        Returns:
            tuple: The two-character country code and full location description for the area.
        """
        country = ''
        location = []
        i = 7   # Counter to avoid potential runaway processing
        while i and area_id and not country:
            i -= 1
            area = self.result_cache[AREA][area_id] if area_id in self.result_cache[AREA] else Area('', '', '', '', '')
            country = area.country
            area_id = area.parent
            if not location or config.setting[OPT_AREA_DETAILS] or area.type not in EXCLUDE_AREA_TYPES:
                location.append(area.name)
        return country, ', '.join(location)


class AdditionalArtistsDetailsOptionsPage(OptionsPage):
    """Options page for the Additional Artists Details plugin.
    """

    NAME = "additional_artists_details"
    TITLE = "Additional Artists Details"
    PARENT = "plugins"

    options = [
        config.BoolOption('setting', OPT_PROCESS_TRACKS, False),
        config.BoolOption('setting', OPT_AREA_DETAILS, False),
    ]

    def __init__(self, parent=None):
        super(AdditionalArtistsDetailsOptionsPage, self).__init__(parent)
        self.ui = Ui_AdditionalArtistsDetailsOptionsPage()
        self.ui.setupUi(self)

        # Enable external link
        self.ui.format_description.setOpenExternalLinks(True)

    def load(self):
        """Load the option settings.
        """
        self.ui.cb_process_tracks.setChecked(config.setting[OPT_PROCESS_TRACKS])
        self.ui.cb_area_details.setChecked(config.setting[OPT_AREA_DETAILS])

    def save(self):
        """Save the option settings.
        """
        # self._set_settings(config.setting)
        config.setting[OPT_PROCESS_TRACKS] = self.ui.cb_process_tracks.isChecked()
        config.setting[OPT_AREA_DETAILS] = self.ui.cb_area_details.isChecked()


plugin = ArtistDetailsPlugin()

# Register the plugin to run at a LOW priority so that other plugins that
# modify the artist information can complete their processing and this plugin
# is working with the latest updated data.
register_album_metadata_processor(plugin.make_album_vars, priority=PluginPriority.LOW)
register_track_metadata_processor(plugin.make_track_vars, priority=PluginPriority.LOW)

register_album_post_removal_processor(plugin.remove_album)
register_options_page(AdditionalArtistsDetailsOptionsPage)



# -*- coding: utf-8 -*-

# Copyright (C) 2016 Anderson Mesquita <andersonvom@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.

from picard import metadata
from picard.util.textencoding import unaccent

PLUGIN_NAME = "Non-ASCII Equivalents"
PLUGIN_AUTHOR = "Anderson Mesquita <andersonvom@trysometinghere>, Konrad Marciniak"
PLUGIN_VERSION = "0.5"
PLUGIN_API_VERSIONS = ["0.9", "0.10", "0.11", "0.15", "2.0"]
PLUGIN_LICENSE = "GPL-3.0-or-later"
PLUGIN_LICENSE_URL = "https://gnu.org/licenses/gpl.html"
PLUGIN_DESCRIPTION = '''Replaces accented and otherwise non-ASCII characters
with a somewhat equivalent version of their ASCII counterparts. This allows old
devices to be able to display song artists and titles somewhat correctly,
instead of displaying weird or blank symbols. It's an attempt to do a little
better than Musicbrainz's native "Replace non-ASCII characters" option.

Currently replaces characters on "album", "albumartist", "albumartists", "albumartistsort", "albumsort", "artist", "artists", "artistsort" and "title" tags.'''

CHAR_TABLE = {
    # Misc Letters
    "Å": "AA",
    "å": "aa",
    "Æ": "AE",
    "æ": "ae",
    "Œ": "OE",
    "œ": "oe",
    "ẞ": "ss",
    "ß": "ss",
    "Ø": "O",
    "ø": "o",
    "Ł": "L",
    "ł": "l",
    "Þ": "Th", # Thorn
    "þ": "th",
    "Ð": "D", # Eth
    "ð": "d",

    # Punctuation
    "¡": "!",
    "¿": "?",
    "–": "--",
    "—": "--",
    "―": "--",
    "«": "<<",
    "»": ">>",
    "‘": "'",
    "’": "'",
    "‚": ",",
    "‛": "'",
    "“": '"',
    "”": '"',
    "„": ",,",
    "‟": '"',
    "‹": "<",
    "›": ">",
    "⹂": ",,",
    "「": "|-",
    "」": "-|",
    "『": "|-",
    "』": "-|",
    "〝": '"',
    "〞": '"',
    "〟": ",,",
    "﹁": "-|",
    "﹂": "|-",
    "﹃": "-|",
    "﹄": "|-",
    "｢": "|-",
    "｣": "-|",
    "・": ".", # Katakana middle dot

    # Mathematics
    "≠": "!=",
    "≤": "<=",
    "≥": ">=",
    "±": "+-",
    "∓": "-+",
    "×": "x",
    "·": ".",
    "÷": "/",
    "√": "\\/",
    "∑": "E",
    "≪": "<<", # these are different
    "≫": ">>", # from the quotation marks

    # Misc
    "°": "o",
    "µ": "u",
    "ı": "i",
    "†": "t",
    "©": "(c)",
    "®": "(R)",
    "♥": "<3",
    "→": "-->",
    "☆": "*",
    "★": "*",
}

FILTER_TAGS = [
    "album",
    "albumartist",
    "albumartists",
    "albumartistsort",
    "albumsort",
    "artist",
    "artists",
    "artistsort",
    "title",
]


def sanitize(char):
    if char in CHAR_TABLE:
        return CHAR_TABLE[char]
    return unaccent(char)


def to_ascii(word):
    return "".join(sanitize(char) for char in word)


def main(tagger, metadata, *args):
    for name, value in metadata.rawitems():
        if name in FILTER_TAGS:
            metadata[name] = [to_ascii(x) for x in value]


metadata.register_track_metadata_processor(main)
metadata.register_album_metadata_processor(main)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 Bob Swift (rdswift)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

PLUGIN_NAME = 'Format Performer Tags'
PLUGIN_AUTHOR = 'Bob Swift, Philipp Wolfer'
PLUGIN_DESCRIPTION = '''
This plugin provides options with respect to the formatting of performer
tags.  It has been developed using the 'Standardise Performers' plugin by
Sophist as the basis for retrieving and processing the performer data for
each of the tracks.  The format of the resulting tags can be customized
in the option settings page.
'''

PLUGIN_VERSION = "0.8.1"
PLUGIN_API_VERSIONS = ["2.0"]
PLUGIN_LICENSE = "GPL-2.0-or-later"
PLUGIN_LICENSE_URL = "https://www.gnu.org/licenses/gpl-2.0.html"

PLUGIN_USER_GUIDE_URL = "https://github.com/rdswift/picard-plugins/blob/2.0_RDS_Plugins/plugins/format_performer_tags/docs/README.md"

import re
from picard import config, log
from picard.metadata import Metadata, register_track_metadata_processor
from picard.plugin import PluginPriority
from picard.ui.options import register_options_page, OptionsPage
from picard.plugins.format_performer_tags.ui_options_format_performer_tags import Ui_FormatPerformerTagsOptionsPage

performers_split = re.compile(r", | and ").split

WORD_LIST = ['guest', 'solo', 'additional']


def get_word_dict(settings):
    word_dict = {}
    for word in WORD_LIST:
        word_dict[word] = settings['format_group_' + word]
    return word_dict


def rewrite_tag(key, values, metadata, word_dict, settings):
    if ':' not in key:
        mainkey = key
        subkey = ''
    else:
        mainkey, subkey = key.split(':', 1)
    log.debug("%s: Removing key: '%s'", PLUGIN_NAME, key,)
    metadata.delete(key)
    log.debug("%s: Formatting Performer [%s: %s]", PLUGIN_NAME, subkey, values,)
    if not subkey:
        instruments = []
    else:
        instruments = performers_split(subkey)
    if instruments:
        for instrument in instruments:
            groups = { 1: [], 2: [], 3: [], 4: [], }
            vocals = ''
            if instrument:
                instrument_key = ''
                words = instrument.split()
                for word in words[:]:
                    if word in WORD_LIST:
                        groups[word_dict[word]].append(word)
                        words.remove(word)
                display_group = {}
                for group_number in range(1, 5):
                    if groups[group_number]:
                        group_separator = settings["format_group_{0}_sep_char".format(group_number)]
                        if not group_separator:
                            group_separator = " "
                        display_group[group_number] = settings["format_group_{0}_start_char".format(group_number)] \
                            + group_separator.join(groups[group_number]) \
                            + settings["format_group_{0}_end_char".format(group_number)]
                    else:
                        display_group[group_number] = ""
                if words:
                    instrument_key = ' '.join(words)
                    if (len(words) > 1) and (words[-1] in ["vocal", "vocals",]):
                        vocals = " ".join(words[:-1])
                        instrument_key = words[-1]
                else:
                    instrument_key = ''
                if vocals:
                    group_number = settings["format_group_vocals"]
                    temp_group = groups[group_number][:]
                    if group_number < 2:
                        temp_group.append(vocals)
                    else:
                        temp_group.insert(0, vocals)
                    group_separator = settings["format_group_{0}_sep_char".format(group_number)]
                    if not group_separator:
                        group_separator = " "
                    display_group[group_number] = settings["format_group_{0}_start_char".format(group_number)] \
                        + group_separator.join(temp_group) \
                        + settings["format_group_{0}_end_char".format(group_number)]
            newkey = ('%s:%s%s%s%s' % (mainkey, display_group[1], instrument_key, display_group[2], display_group[3],))
            log.debug("%s: newkey: %s", PLUGIN_NAME, newkey,)
            for value in values:
                metadata.add_unique(newkey, (value + display_group[4]))
    else:
        newkey = '%s:' % (mainkey,)
        log.debug("%s: newkey: %s", PLUGIN_NAME, newkey,)
        for value in values:
            metadata.add_unique(newkey, value)


def format_performer_tags(album, metadata, *args):
    word_dict = get_word_dict(config.setting)
    for key, values in list(filter(lambda filter_tuple: filter_tuple[0].startswith('performer') or filter_tuple[0].startswith('~performersort'), metadata.rawitems())):
        rewrite_tag(key, values, metadata, word_dict, config.setting)


class FormatPerformerTagsOptionsPage(OptionsPage):

    NAME = "format_performer_tags"
    TITLE = "Format Performer Tags"
    PARENT = "plugins"
    HELP_URL = "https://github.com/metabrainz/picard-plugins/blob/2.0/plugins/format_performer_tags/docs/README.md"

    options = [
        config.IntOption("setting", "format_group_additional", 3),
        config.IntOption("setting", "format_group_guest", 4),
        config.IntOption("setting", "format_group_solo", 3),
        config.IntOption("setting", "format_group_vocals", 2),
        config.TextOption("setting", "format_group_1_start_char", ''),
        config.TextOption("setting", "format_group_1_end_char", ' '),
        config.TextOption("setting", "format_group_1_sep_char", ''),
        config.TextOption("setting", "format_group_2_start_char", ', '),
        config.TextOption("setting", "format_group_2_end_char", ''),
        config.TextOption("setting", "format_group_2_sep_char", ''),
        config.TextOption("setting", "format_group_3_start_char", ' ('),
        config.TextOption("setting", "format_group_3_end_char", ')'),
        config.TextOption("setting", "format_group_3_sep_char", ''),
        config.TextOption("setting", "format_group_4_start_char", ' ('),
        config.TextOption("setting", "format_group_4_end_char", ')'),
        config.TextOption("setting", "format_group_4_sep_char", ''),
    ]

    def __init__(self, parent=None):
        super(FormatPerformerTagsOptionsPage, self).__init__(parent)
        self.ui = Ui_FormatPerformerTagsOptionsPage()
        self.ui.setupUi(self)
        self.ui.additional_rb_1.clicked.connect(self.update_examples)
        self.ui.additional_rb_2.clicked.connect(self.update_examples)
        self.ui.additional_rb_3.clicked.connect(self.update_examples)
        self.ui.additional_rb_4.clicked.connect(self.update_examples)
        self.ui.guest_rb_1.clicked.connect(self.update_examples)
        self.ui.guest_rb_2.clicked.connect(self.update_examples)
        self.ui.guest_rb_3.clicked.connect(self.update_examples)
        self.ui.guest_rb_4.clicked.connect(self.update_examples)
        self.ui.solo_rb_1.clicked.connect(self.update_examples)
        self.ui.solo_rb_2.clicked.connect(self.update_examples)
        self.ui.solo_rb_3.clicked.connect(self.update_examples)
        self.ui.solo_rb_4.clicked.connect(self.update_examples)
        self.ui.vocals_rb_1.clicked.connect(self.update_examples)
        self.ui.vocals_rb_2.clicked.connect(self.update_examples)
        self.ui.vocals_rb_3.clicked.connect(self.update_examples)
        self.ui.vocals_rb_4.clicked.connect(self.update_examples)
        self.ui.format_group_1_start_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_2_start_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_3_start_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_4_start_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_1_sep_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_2_sep_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_3_sep_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_4_sep_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_1_end_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_2_end_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_3_end_char.editingFinished.connect(self.update_examples)
        self.ui.format_group_4_end_char.editingFinished.connect(self.update_examples)

    def load(self):
        # Enable external link
        self.ui.format_description.setOpenExternalLinks(True)

        # Settings for Keyword: additional
        temp = config.setting["format_group_additional"]
        if temp > 3:
            self.ui.additional_rb_4.setChecked(True)
        elif temp > 2:
            self.ui.additional_rb_3.setChecked(True)
        elif temp > 1:
            self.ui.additional_rb_2.setChecked(True)
        else:
            self.ui.additional_rb_1.setChecked(True)

        # Settings for Keyword: guest
        temp = config.setting["format_group_guest"]
        if temp > 3:
            self.ui.guest_rb_4.setChecked(True)
        elif temp > 2:
            self.ui.guest_rb_3.setChecked(True)
        elif temp > 1:
            self.ui.guest_rb_2.setChecked(True)
        else:
            self.ui.guest_rb_1.setChecked(True)

        # Settings for Keyword: solo
        temp = config.setting["format_group_solo"]
        if temp > 3:
            self.ui.solo_rb_4.setChecked(True)
        elif temp > 2:
            self.ui.solo_rb_3.setChecked(True)
        elif temp > 1:
            self.ui.solo_rb_2.setChecked(True)
        else:
            self.ui.solo_rb_1.setChecked(True)

        # Settings for all vocal keywords
        temp = config.setting["format_group_vocals"]
        if temp > 3:
            self.ui.vocals_rb_4.setChecked(True)
        elif temp > 2:
            self.ui.vocals_rb_3.setChecked(True)
        elif temp > 1:
            self.ui.vocals_rb_2.setChecked(True)
        else:
            self.ui.vocals_rb_1.setChecked(True)

        # Settings for word group 1
        self.ui.format_group_1_start_char.setText(config.setting["format_group_1_start_char"])
        self.ui.format_group_1_end_char.setText(config.setting["format_group_1_end_char"])
        self.ui.format_group_1_sep_char.setText(config.setting["format_group_1_sep_char"])

        # Settings for word group 2
        self.ui.format_group_2_start_char.setText(config.setting["format_group_2_start_char"])
        self.ui.format_group_2_end_char.setText(config.setting["format_group_2_end_char"])
        self.ui.format_group_2_sep_char.setText(config.setting["format_group_2_sep_char"])

        # Settings for word group 3
        self.ui.format_group_3_start_char.setText(config.setting["format_group_3_start_char"])
        self.ui.format_group_3_end_char.setText(config.setting["format_group_3_end_char"])
        self.ui.format_group_3_sep_char.setText(config.setting["format_group_3_sep_char"])

        # Settings for word group 4
        self.ui.format_group_4_start_char.setText(config.setting["format_group_4_start_char"])
        self.ui.format_group_4_end_char.setText(config.setting["format_group_4_end_char"])
        self.ui.format_group_4_sep_char.setText(config.setting["format_group_4_sep_char"])
        self.update_examples()

        # TODO: Modify self.format_description in ui_options_format_performer_tags.py to include a placeholder
        #       such as {user_guide_url} so that the translated string can be formatted to include the value
        #       of PLUGIN_USER_GUIDE_URL to dynamically set the link while not requiring retranslation if the
        #       link changes.  Preliminary code something like:
        #
        #       temp = (self.ui.format_description.text).format(user_guide_url=PLUGIN_USER_GUIDE_URL,)
        #       self.ui.format_description.setText(temp)

    def save(self):
        self._set_settings(config.setting)

    def restore_defaults(self):
        super().restore_defaults()
        self.update_examples()

    def _set_settings(self, settings):

        # Process 'additional' keyword settings
        temp = 1
        if self.ui.additional_rb_2.isChecked(): temp = 2
        if self.ui.additional_rb_3.isChecked(): temp = 3
        if self.ui.additional_rb_4.isChecked(): temp = 4
        settings["format_group_additional"] = temp

        # Process 'guest' keyword settings
        temp = 1
        if self.ui.guest_rb_2.isChecked(): temp = 2
        if self.ui.guest_rb_3.isChecked(): temp = 3
        if self.ui.guest_rb_4.isChecked(): temp = 4
        settings["format_group_guest"] = temp

        # Process 'solo' keyword settings
        temp = 1
        if self.ui.solo_rb_2.isChecked(): temp = 2
        if self.ui.solo_rb_3.isChecked(): temp = 3
        if self.ui.solo_rb_4.isChecked(): temp = 4
        settings["format_group_solo"] = temp

        # Process all vocal keyword settings
        temp = 1
        if self.ui.vocals_rb_2.isChecked(): temp = 2
        if self.ui.vocals_rb_3.isChecked(): temp = 3
        if self.ui.vocals_rb_4.isChecked(): temp = 4
        settings["format_group_vocals"] = temp

        # Settings for word group 1
        settings["format_group_1_start_char"] = self.ui.format_group_1_start_char.text()
        settings["format_group_1_end_char"] = self.ui.format_group_1_end_char.text()
        settings["format_group_1_sep_char"] = self.ui.format_group_1_sep_char.text()

        # Settings for word group 2
        settings["format_group_2_start_char"] = self.ui.format_group_2_start_char.text()
        settings["format_group_2_end_char"] = self.ui.format_group_2_end_char.text()
        settings["format_group_2_sep_char"] = self.ui.format_group_2_sep_char.text()

        # Settings for word group 3
        settings["format_group_3_start_char"] = self.ui.format_group_3_start_char.text()
        settings["format_group_3_end_char"] = self.ui.format_group_3_end_char.text()
        settings["format_group_3_sep_char"] = self.ui.format_group_3_sep_char.text()

        # Settings for word group 4
        settings["format_group_4_start_char"] = self.ui.format_group_4_start_char.text()
        settings["format_group_4_end_char"] = self.ui.format_group_4_end_char.text()
        settings["format_group_4_sep_char"] = self.ui.format_group_4_sep_char.text()

    def update_examples(self):
        settings = {}
        self._set_settings(settings)
        word_dict = get_word_dict(settings)

        instruments_credits = {
            "guitar": ["Johnny Flux", "John Watson"],
            "guest guitar": ["Jimmy Page"],
            "additional guest solo guitar": ["Jimmy Page"],
        }
        instruments_example = self.build_example(instruments_credits, word_dict, settings)
        self.ui.example_instruments.setText(instruments_example)

        vocals_credits = {
            "additional solo lead vocals": ["Robert Plant"],
            "additional solo guest lead vocals": ["Sandy Denny"],
        }
        vocals_example = self.build_example(vocals_credits, word_dict, settings)
        self.ui.example_vocals.setText(vocals_example)

    @staticmethod
    def build_example(credits, word_dict, settings):
        prefix = "performer:"
        metadata = Metadata()
        for key, values in credits.items():
            rewrite_tag(prefix + key, values, metadata, word_dict, settings)

        examples = []
        for key, values in metadata.rawitems():
            credit = "%s: %s" % (key, ", ".join(values))
            if credit.startswith(prefix):
                credit = credit[len(prefix):]
            examples.append(credit)
        return "\n".join(examples)


# Register the plugin to run at a HIGH priority.
register_track_metadata_processor(format_performer_tags, priority=PluginPriority.HIGH)
register_options_page(FormatPerformerTagsOptionsPage)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2021 Philipp Wolfer
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

PLUGIN_NAME = 'fanart.tv cover art'
PLUGIN_AUTHOR = 'Philipp Wolfer, Sambhav Kothari'
PLUGIN_DESCRIPTION = ('Use cover art from fanart.tv.<br /><br />'
                      'To use this plugin you have to register a personal API key on '
                      '<a href="https://fanart.tv/get-an-api-key/">fanart.tv</a>.')
PLUGIN_VERSION = "1.6.3"
PLUGIN_API_VERSIONS = ["2.0", "2.1", "2.2", "2.3", "2.4", "2.5", "2.6"]
PLUGIN_LICENSE = "GPL-2.0-or-later"
PLUGIN_LICENSE_URL = "https://www.gnu.org/licenses/gpl-2.0.html"

from functools import partial
from PyQt5.QtCore import QUrl
from PyQt5.QtNetwork import QNetworkReply
from picard import config, log
from picard.coverart.providers import (
    CoverArtProvider,
    ProviderOptions,
    register_cover_art_provider,
)
from picard.coverart.image import CoverArtImage
from picard.config import TextOption
from .ui_options_fanarttv import Ui_FanartTvOptionsPage

FANART_HOST = "webservice.fanart.tv"
FANART_PORT = 443
FANART_APIKEY = "21305dd1589766f4d544535ad4df12f4"

OPTION_CDART_ALWAYS = "always"
OPTION_CDART_NEVER = "never"
OPTION_CDART_NOALBUMART = "noalbumart"


def cover_sort_key(cover):
    """For sorting a list of cover arts by likes."""
    try:
        return int(cover["likes"]) if "likes" in cover else 0
    except ValueError:
        return 0


def encode_queryarg(arg):
    return bytes(QUrl.toPercentEncoding(arg)).decode()


class FanartTvOptionsPage(ProviderOptions):

    _options_ui = Ui_FanartTvOptionsPage

    options = [
        TextOption("setting", "fanarttv_client_key", ""),
        TextOption("setting", "fanarttv_use_cdart", OPTION_CDART_NOALBUMART),
    ]

    def load(self):
        setting = config.setting
        self.ui.fanarttv_client_key.setText(setting["fanarttv_client_key"])
        if setting["fanarttv_use_cdart"] == OPTION_CDART_ALWAYS:
            self.ui.fanarttv_cdart_use_always.setChecked(True)
        elif setting["fanarttv_use_cdart"] == OPTION_CDART_NEVER:
            self.ui.fanarttv_cdart_use_never.setChecked(True)
        elif setting["fanarttv_use_cdart"] == OPTION_CDART_NOALBUMART:
            self.ui.fanarttv_cdart_use_if_no_albumcover.setChecked(True)

    def save(self):
        setting = config.setting
        setting["fanarttv_client_key"] = self.ui.fanarttv_client_key.text()
        if self.ui.fanarttv_cdart_use_always.isChecked():
            setting["fanarttv_use_cdart"] = OPTION_CDART_ALWAYS
        elif self.ui.fanarttv_cdart_use_never.isChecked():
            setting["fanarttv_use_cdart"] = OPTION_CDART_NEVER
        elif self.ui.fanarttv_cdart_use_if_no_albumcover.isChecked():
            setting["fanarttv_use_cdart"] = OPTION_CDART_NOALBUMART


class FanartTvCoverArtImage(CoverArtImage):

    """Image from fanart.tv"""

    support_types = True
    sourceprefix = "FATV"


class CoverArtProviderFanartTv(CoverArtProvider):

    """Use fanart.tv to get cover art"""

    NAME = "fanart.tv"
    TITLE = "fanart.tv"
    OPTIONS = FanartTvOptionsPage

    def enabled(self):
        return (self._client_key and super().enabled()
                and not self.coverart.front_image_found)

    def queue_images(self):
        release_group_id = self.metadata["musicbrainz_releasegroupid"]
        path = "/v3/music/albums/%s" % (release_group_id, )
        queryargs = {
            "api_key": encode_queryarg(FANART_APIKEY),
            "client_key": encode_queryarg(self._client_key),
        }
        log.debug("CoverArtProviderFanartTv.queue_downloads: %s" % path)
        self.album.tagger.webservice.get(
            FANART_HOST,
            FANART_PORT,
            path,
            partial(self._json_downloaded, release_group_id),
            priority=True,
            important=False,
            parse_response_type='json',
            queryargs=queryargs)
        self.album._requests += 1
        return CoverArtProvider.WAIT

    @property
    def _client_key(self):
        return config.setting["fanarttv_client_key"]

    def _json_downloaded(self, release_group_id, data, reply, error):
        self.album._requests -= 1

        if error:
            if error != QNetworkReply.ContentNotFoundError:
                error_level = log.error
            else:
                error_level = log.debug
            error_level("Problem requesting metadata in fanart.tv plugin: %s",
                        error)
        else:
            try:
                release = data["albums"][release_group_id]
                has_cover = "albumcover" in release
                has_cdart = "cdart" in release
                use_cdart = config.setting["fanarttv_use_cdart"]

                if has_cover:
                    covers = release["albumcover"]
                    self._select_and_add_cover_art(covers, ["front"])

                if has_cdart and (use_cdart == OPTION_CDART_ALWAYS
                                  or (use_cdart == OPTION_CDART_NOALBUMART
                                      and not has_cover)):
                    covers = release["cdart"]
                    types = ["medium"]
                    if not has_cover:
                        types.append("front")
                    self._select_and_add_cover_art(covers, types)
            except (AttributeError, KeyError, TypeError):
                log.error("Problem processing downloaded metadata in fanart.tv plugin", exc_info=True)

        self.next_in_queue()

    def _select_and_add_cover_art(self, covers, types):
        covers = sorted(covers, key=cover_sort_key, reverse=True)
        url = covers[0]["url"]
        log.debug("CoverArtProviderFanartTv found artwork %s" % url)
        self.queue_put(FanartTvCoverArtImage(url, types=types))


register_cover_art_provider(CoverArtProviderFanartTv)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Flaky
# Copyright (C) 2023 Bob Swift (rdswift)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

PLUGIN_NAME = "Submit Folksonomy Tags"
PLUGIN_AUTHOR = "Flaky"
PLUGIN_DESCRIPTION = """
A plugin allowing submission of specific tags on tracks you own (defaults to <i>genre</i> and <i>mood</i>) as folksonomy tags on MusicBrainz. Supports submitting to recording, release, release group and release artist entities.

A MusicBrainz login is required to use this plugin. Log in first by going to the General options. Then, to use, right click on a track or release then go to <i>Plugins</i> and depending on what you want to submit, choose the option you want.

Uses code from rdswift's "Submit ISRC" plugin (specifically, the handling of the network response)
"""
PLUGIN_VERSION = '0.3'
PLUGIN_API_VERSIONS = ['2.2', '2.9']
PLUGIN_LICENSE = "GPL-2.0"
PLUGIN_LICENSE_URL = "https://www.gnu.org/licenses/gpl-2.0.txt"

from picard import config, log, PICARD_VERSION
from picard.album import Album, Track
from picard.ui.itemviews import (BaseAction,
                                 register_album_action,
                                 register_track_action
                                 )
from picard.ui.options import OptionsPage, register_options_page
from picard.version import Version
from picard.webservice.api_helpers import MBAPIHelper, _wrap_xml_metadata
from .ui_config import TagSubmitPluginOptionsUI
import re
import functools
from xml.sax.saxutils import escape
from PyQt5 import QtCore
from PyQt5.QtWidgets import QMessageBox

NEW_MBAPIHelper = (PICARD_VERSION >= Version(2, 9, 0, 'beta', 2))

# List of Qt network error codes.
# From "Submit ISRC" plugin - credit to rdswift.
q_error_codes = {
    0: 'No error',
    1: "The remote server refused the connection (the server is not accepting requests).",
    2: "The remote server closed the connection prematurely, before the entire reply was received and processed.",
    3: "The remote host name was not found (invalid hostname).",
    4: "The connection to the remote server timed out.",
    5: "The operation was canceled via calls to abort() or close() before it was finished.",
    6: "The SSL/TLS handshake failed and the encrypted channel could not be established. The sslErrors() signal should have been emitted.",
    7: "The connection was broken due to disconnection from the network, however the system has initiated roaming to another access point. The request should be resubmitted and will be processed as soon as the connection is re-established.",
    8: "The connection was broken due to disconnection from the network or failure to start the network.",
    9: "The background request is not currently allowed due to platform policy.",
    10: "While following redirects, the maximum limit was reached.",
    11: "While following redirects, the network access API detected a redirect from a encrypted protocol (https) to an unencrypted one (http).",
    99: "An unknown network-related error was detected.",
    101: "The connection to the proxy server was refused (the proxy server is not accepting requests).",
    102: "The proxy server closed the connection prematurely, before the entire reply was received and processed.",
    103: "The proxy host name was not found (invalid proxy hostname).",
    104: "The connection to the proxy timed out or the proxy did not reply in time to the request sent.",
    105: "The proxy requires authentication in order to honour the request but did not accept any credentials offered (if any).",
    199: "An unknown proxy-related error was detected.",
    201: "The access to the remote content was denied (similar to HTTP error 403).",
    202: "The operation requested on the remote content is not permitted.",
    203: "The remote content was not found at the server (similar to HTTP error 404).",
    204: "The remote server requires authentication to serve the content but the credentials provided were not accepted (if any).",
    205: "The request needed to be sent again, but this failed for example because the upload data could not be read a second time.",
    206: "The request could not be completed due to a conflict with the current state of the resource.",
    207: "The requested resource is no longer available at the server.",
    299: "An unknown error related to the remote content was detected.",
    301: "The Network Access API cannot honor the request because the protocol is not known.",
    302: "The requested operation is invalid for this protocol.",
    399: "A breakdown in protocol was detected (parsing error, invalid or unexpected responses, etc.).",
    401: "The server encountered an unexpected condition which prevented it from fulfilling the request.",
    402: "The server does not support the functionality required to fulfill the request.",
    403: "The server is unable to handle the request at this time.",
    499: "An unknown error related to the server response was detected.",
}

# Some internal settings.
# Don't change these unless you know what you're doing.
# You can change the tags you want to submit in the settings.
client_params = {
    "client": f"picard_plugin_{PLUGIN_NAME.replace(' ', '_')}-v{PLUGIN_VERSION}"
}
default_tags_to_submit = ['genre', 'mood']

# The options as saved in Picard.ini
config.BoolOption("setting", 'tag_submit_plugin_destructive', False)
config.BoolOption("setting", 'tag_submit_plugin_destructive_alert_acknowledged', False)
config.BoolOption("setting", 'tag_submit_plugin_aliases_enabled', False)
config.ListOption("setting", 'tag_submit_plugin_alias_list', [])
config.ListOption("setting", 'tag_submit_plugin_tags_to_submit', default_tags_to_submit)

def tag_submit_handler(document, reply, error, tagger):
    """
    The function handling the network response from MusicBrainz
    or QtNetwork, showing a message box if an error had occurred.

    Uses the network response handler code from rdswift's "Submit ISRC"
    plugin.
    """
    if error:
        # Error handling from rdswift's Submit ISRC plugin
        xml_text = str(document, 'UTF-8') if isinstance(document, (bytes, bytearray, QtCore.QByteArray)) else str(document)

        # Build error text message from returned xml payload
        err_text = ''
        matches = re.findall(r'<text>(.*?)</text>', xml_text)
        if matches:
            err_text = '\n'.join(matches)
        else:
            err_text = ''

        if not err_text:
            err_text = q_error_codes[error] if error in q_error_codes else 'There was no error message provided.'

        error = QMessageBox()
        error.setStandardButtons(QMessageBox.Ok)
        error.setDefaultButton(QMessageBox.Ok)
        error.setIcon(QMessageBox.Critical)
        error.setText(f"<p>An error has occurred submitting the tags to MusicBrainz.</p><p>{err_text}</p>")
        error.exec_()
    else:
        tagger.window.set_statusbar_message(
            "Successfully submitted tags to MusicBrainz."
            )

def process_tag_aliases(tag_input):
    """
    Retrieves a string as input, and searches the tag alias tuple list
    for a match.
    """
    matched_tag_index = next(
        (tag for tag,
            tag_tuple in enumerate(config.setting['tag_submit_plugin_alias_list'])
            if tag_tuple[0] == tag_input.lower()),
        None
    )
    if matched_tag_index is not None:
        resolved_tag = config.setting['tag_submit_plugin_alias_list'][matched_tag_index][1]
        return resolved_tag
    else:
        return tag_input

def process_objs_to_track_list(objs):
    """
    Creates a track list out of Album/Track objects
    """
    track_list = []
    for item in objs:
        if isinstance(item, Track):
            track_list.append(item)
        elif isinstance(item, Album):
            if len(item.tracks) > 0:
                for track in item.tracks:
                    track_list.append(track)
    return track_list

# TODO handle artist
def handle_submit_process(tagger, track_list, target_tag):
    """
    Does some pre-processing before submitting tags. Handles tag deduplication
    and halting when inconsistent tagging is detected (i.e. the user is trying
    to submit tags to a release with the submitted track tags not being
    consistent.)
    """

    dict_key = ""
    tags_to_search = config.setting['tag_submit_plugin_tags_to_submit']

    # Variable to enable when inconsistent tagging is detected, which can be problematic for anything other than recordings.
    alert_inconsistent = True
    inconsistent_detected = False
    # Variable to enable alert if multiple MBIDs are associated, must be toggled.
    alert_multiple_mbids = False

    # TODO when Windows Picard updates with Python 3.10, use case/switch.
    if target_tag == "musicbrainz_recordingid":
        dict_key = "recording"
        alert_inconsistent = False
    elif target_tag == "musicbrainz_albumid":
        dict_key = "release"
    elif target_tag == "musicbrainz_releasegroupid":
        dict_key = "release-group"
    elif target_tag == "musicbrainz_albumartistid" or target_tag == "musicbrainz_artistid":
        dict_key = "artist"

    data = {dict_key: {}}

    last_tags = {"mbid": ""}
    banned_mbids = {
        # Any artist entities that can be applied to multiple artists go here.
        # SPAs generally fit the bill here.
        "f731ccc4-e22a-43af-a747-64213329e088", # artist: [anonymous]
        "33cf029c-63b0-41a0-9855-be2a3665fb3b", # artist: [data]
        "314e1c25-dde7-4e4d-b2f4-0a7b9f7c56dc", # artist: [dialogue]
        "eec63d3c-3b81-4ad4-b1e4-7c147d4d2b61", # artist: [no artist]
        "9be7f096-97ec-4615-8957-8d40b5dcbc41", # artist: [traditional]
        "125ec42a-7229-4250-afc5-e057484327fe", # artist: [unknown]
        "89ad4ac3-39f7-470e-963a-56509c546377", # artist: Various Artists
        "66ea0139-149f-4a0c-8fbf-5ea9ec4a6e49", # artist: [Disney]
        "a0ef7e1d-44ff-4039-9435-7d5fefdeecc9", # artist: [theatre]
        "80a8851f-444c-4539-892b-ad2a49292aa9", # artist: [language instruction]
        "",                                     # blank
    }

    for track in track_list:
        if track.files:
            for file in track.files:
                mbid_list = file.metadata.getall(target_tag)
                if len(mbid_list) > 1:
                    alert_multiple_mbids = True
                for mbid in mbid_list:
                    if mbid not in banned_mbids:
                        processed_tags = []
                        for tag in tags_to_search:
                            if file.metadata[tag]:
                                if tag not in last_tags:
                                    pass
                                else:
                                    # Flip the switch when current tag didn't match last tag on current mbid, on an entity that needs this checked.
                                    if (last_tags[tag] != file.metadata[tag]) and (last_tags["mbid"] == file.metadata[target_tag]) and alert_inconsistent:
                                        inconsistent_detected = True
                                # in any case, process the tags in case the user intends to go with it.
                                processed_tags.extend([tag.strip().lower() for tag
                                                    in re.split(";|/|,", file.metadata[tag])])
                                last_tags[tag] = file.metadata[tag]
                                last_tags["mbid"] = file.metadata[target_tag]
                        # If a track has multiple files associated to it, there may be duplicate tags.
                        processed_tags = list(set(processed_tags))
                        if processed_tags and mbid in data[dict_key]:
                            data[dict_key][mbid].extend(processed_tags)
                        else:
                            data[dict_key][mbid] = processed_tags
                    else:
                        log.info(f"Not submitting MBID {track.metadata[target_tag]} as it was found on 'do not submit' MBID set.")

    # Send an alert when, at the end of it all, inconsistent tagging was detected.
    if inconsistent_detected or alert_multiple_mbids:
        warning = QMessageBox()
        warning.setStandardButtons(QMessageBox.Ok|QMessageBox.Cancel)
        warning.setDefaultButton(QMessageBox.Cancel)
        warning.setIcon(QMessageBox.Warning)
        if inconsistent_detected and alert_multiple_mbids:
            warning.setText("""
            <p><b>WARNING: INCONSISTENT TAGGING AND SUBMISSION TO MULTIPLE MBIDS DETECTED.</b></p>
            <p>You are trying to apply different tags to multiple MusicBrainz entities.</p>
            <p>This isn't a use case this plugin supports whatsoever due to the potential for
            wrong tags to be unintentionally assigned, but detects and warns just in case.</p>
            <p>If this was intentional, click OK. Otherwise, click Cancel.</p>
            """)
        elif inconsistent_detected:
            warning.setText("""
            <p><b>WARNING: INCONSISTENT TAGGING DETECTED.</b></p>
            <p>You are trying to apply multiple tags to one entity, which benefits more from
            having the same tags across all tracks when submitting tags via this plugin.</p>
            <p>If you intended to have tracks in a release to have different submitted tags,
            it's better to cancel this attempt and choose the recording option.
            If you didn't, you should apply the same tag across all tracks.</p>
            <p>If this was intentional, click OK. Otherwise, click Cancel.</p>
            """)
        elif alert_multiple_mbids:
            warning.setText("""
            <p><b>MULTIPLE MBIDS DETECTED.</b></p>
            <p>You are trying to apply a tag to multiple MusicBrainz entities.</p>
            <p>This isn't a use case this plugin supports whatsoever due to the potential for
            wrong tags to be unintentionally assigned, but detects and warns just in case.</p>
            <p>If this was intentional, click OK. Otherwise, click Cancel.</p>
            """)
        result = warning.exec_()
        if result == QMessageBox.Ok:
            upload_tags_to_mbz(data, tagger)
        else:
            tagger.window.set_statusbar_message(
                "Tag submission halted by user request."
            )
    else:
        upload_tags_to_mbz(data, tagger)

def upload_tags_to_mbz(data, tagger):
    """
    Generates the XML from the data retrieved, and then uploads it to MusicBrainz.
    """
    helper = MBAPIHelper(tagger.webservice)

    empty_data = {
        "<recording-list></recording-list>",
        "<release-list></release-list>",
        "<release-group-list></release-group-list>",
        "<artist-list></artist-list>"
    }

    xml_data = []
    upvote_tag_fill = ' vote="upvote"' if not config.setting['tag_submit_plugin_destructive'] else ''
    for key in data:
        # start the list
        xml_data.append(f"<{key}-list>")
        for mbid in data[key]:
            # deduplicate the list of genres so no redundant tags are sent.
            data[key][mbid] = list(set(data[key][mbid]))
            # start the user tag list
            xml_data.extend([f'<{key} id="{mbid}">', "<user-tag-list>"])
            # add the tags
            for tag in data[key][mbid]:
                xml_data.append(f'<user-tag{upvote_tag_fill}><name>{escape(process_tag_aliases(tag.lower()))}</name></user-tag>')
            # close the user tag list
            xml_data.extend(["</user-tag-list>", f"</{key}>"])
        # close the list
        xml_data.append(f"</{key}-list>")
    # make the string that would become our submitted XML.
    final_xml = ''.join(xml_data)
    log.info(final_xml)

    if final_xml not in empty_data:
    # post it to MusicBrainz
        tagger.window.set_statusbar_message(
                "Submitting tags to MusicBrainz..."
                )
        submitted_xml = _wrap_xml_metadata(''.join(xml_data))
        path = '/tag' if NEW_MBAPIHelper else ['tag']
        helper.post(
            path,
            submitted_xml,
            functools.partial(tag_submit_handler, tagger=tagger),
            priority=True,
            queryargs=client_params,
            parse_response_type="xml",
            request_mimetype="application/xml; charset=utf-8"
        )
    else:
        tagger.window.set_statusbar_message(
            "Not submitting to MusicBrainz due to empty data."
            )


class TagSubmitPlugin_OptionsPage(OptionsPage):
    NAME = "tag_submit_plugin"
    TITLE = "Tag Submission Plugin"
    PARENT = "plugins"
    HELP_URL = ""

    def __init__(self, parent=None):
        super().__init__()
        self.ui = TagSubmitPluginOptionsUI(self)
        self.destructive_acknowledgement = config.setting['tag_submit_plugin_destructive_alert_acknowledged']
        self.ui.overwrite_radio_button.clicked.connect(self.on_destructive_selected)

    def on_destructive_selected(self):
        if not config.setting['tag_submit_plugin_destructive_alert_acknowledged']:
            warning = QMessageBox()
            warning.setStandardButtons(QMessageBox.Ok)
            warning.setDefaultButton(QMessageBox.Ok)
            warning.setIcon(QMessageBox.Warning)
            warning.setText("<p><b>WARNING: BY SELECTING TO OVERWRITE ALL TAGS, THIS MEANS <i>ALL</i> TAGS.</b></p><p>By enabling this option, you acknowledge that you may lose the tags already saved online from the tracks you process via this plugin. This alert will only be displayed once before you save.</p><p>If you do not want this behaviour, select the maintain option.</p>")
            warning.exec_()
            config.setting['tag_submit_plugin_destructive_alert_acknowledged'] = True

    def load(self):
        # Destructive option
        if config.setting['tag_submit_plugin_destructive']:
            self.ui.overwrite_radio_button.setChecked(True)
        else:
            self.ui.keep_radio_button.setChecked(True)

        self.ui.tags_to_save_textbox.setText(
            '; '.join(config.setting['tag_submit_plugin_tags_to_submit'])
            )

        # Aliases enabled option
        self.ui.tag_alias_groupbox.setChecked(
            config.setting['tag_submit_plugin_aliases_enabled']
        )

        # Alias list
        if 'tag_submit_plugin_alias_list' in config.setting:
            log.info("Alias list exists! Let's populate the table.")
            for alias_tuple in config.setting['tag_submit_plugin_alias_list']:
                self.ui.add_row(alias_tuple[0], alias_tuple[1])
            self.ui.tag_alias_table.resizeColumnsToContents()

        config.setting['tag_submit_plugin_destructive_alert_acknowledged'] = self.destructive_acknowledgement

    def save(self):
        config.setting['tag_submit_plugin_destructive'] = self.ui.overwrite_radio_button.isChecked()
        config.setting['tag_submit_plugin_aliases_enabled'] = self.ui.tag_alias_groupbox.isChecked()

        tag_textbox_text = self.ui.tags_to_save_textbox.text()
        if tag_textbox_text:
            config.setting['tag_submit_plugin_tags_to_submit'] = [
                tag.strip() for tag in tag_textbox_text.split(';')
                ]
        else:
            config.setting['tag_submit_plugin_tags_to_submit'] = default_tags_to_submit

        if config.setting['tag_submit_plugin_aliases_enabled']:
            new_alias_list = self.ui.rows_to_tuple_list()
            log.info(new_alias_list)
            config.setting['tag_submit_plugin_alias_list'] = new_alias_list

class SubmitTrackTagsMenuAction(BaseAction):
    NAME = 'Submit tags to MusicBrainz (recording)'

    def callback(self, objs):
        handle_submit_process(
            objs[0].tagger,
            process_objs_to_track_list(objs),
            "musicbrainz_recordingid"
            )

class SubmitReleaseTagsMenuAction(BaseAction):
    NAME = 'Submit tags to MusicBrainz (release)'

    def callback(self, objs):
        handle_submit_process(
            objs[0].tagger,
            process_objs_to_track_list(objs),
            "musicbrainz_albumid"
            )

class SubmitRGTagsMenuAction(BaseAction):
    NAME = 'Submit tags to MusicBrainz (release group)'

    def callback(self, objs):
        handle_submit_process(
            objs[0].tagger,
            process_objs_to_track_list(objs),
            "musicbrainz_releasegroupid"
            )

class SubmitRATagsMenuAction(BaseAction):
    NAME = 'Submit tags to MusicBrainz (release artist)'

    def callback(self, objs):
        handle_submit_process(
            objs[0].tagger,
            process_objs_to_track_list(objs),
            "musicbrainz_albumartistid"
            )

register_options_page(TagSubmitPlugin_OptionsPage)
register_album_action(SubmitTrackTagsMenuAction())
register_track_action(SubmitTrackTagsMenuAction())
register_album_action(SubmitReleaseTagsMenuAction())
register_track_action(SubmitReleaseTagsMenuAction())
register_album_action(SubmitRGTagsMenuAction())
register_track_action(SubmitRGTagsMenuAction())
register_album_action(SubmitRATagsMenuAction())
register_track_action(SubmitRATagsMenuAction())



# -*- coding: utf-8 -*-
"""
Declare constants for Picard Classical Extras plugin
v2.0.2
"""
# Copyright (C) 2018 Mark Evens
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

RELATION_TYPES = {
    'work': [
        'arranger',
        'instrument arranger',
        'orchestrator',
        'composer',
        'writer',
        'lyricist',
        'librettist',
        'revised by',
        'translator',
        'reconstructed by',
        'vocal arranger'],
    'release': [
        'instrument',
        'performer',
        'vocal',
        'performing orchestra',
        'conductor',
        'chorus master',
        'concertmaster',
        'arranger',
        'instrument arranger',
        'orchestrator',
        'vocal arranger'],
    'recording': [
        'instrument',
        'performer',
        'vocal',
        'performing orchestra',
        'conductor',
        'chorus master',
        'concertmaster',
        'arranger',
        'instrument arranger',
        'orchestrator',
        'vocal arranger']}

ARTISTS_OPTIONS = [
    {'option': 'classical_extra_artists',
     'name': 'run extra artists',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_orchestras',
     'name': 'orchestra strings',
     'type': 'Text',
     'default': 'orchestra, philharmonic, philharmonica, philharmoniker, musicians, academy, symphony, orkester'
     },
    {'option': 'cea_choirs',
     'name': 'choir strings',
     'type': 'Text',
     'default': 'choir, choir vocals, chorus, singers, domchors, domspatzen, koor, kammerkoor'
     },
    {'option': 'cea_groups',
     'name': 'group strings',
     'type': 'Text',
     'default': 'ensemble, band, group, trio, quartet, quintet, sextet, septet, octet, chamber, consort, players, '
                'les ,the , quartett'
     },
    {'option': 'cea_aliases',
     'name': 'replace artist name with alias?',
     'value': 'replace',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_aliases_composer',
     'name': 'replace artist name with alias?',
     'value': 'composer',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_no_aliases',
     'name': 'replace artist name with alias?',
     'value': 'no replace',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_alias_overrides',
     'name': 'alias vs credited-as',
     'value': 'alias over-rides',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_credited_overrides',
     'name': 'alias vs credited-as',
     'value': 'credited-as over-rides',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_ra_use',
     'name': 'use recording artist',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_ra_trackartist',
     'name': 'recording artist name style',
     'value': 'track artist',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_ra_performer',
     'name': 'recording artist name style',
     'value': 'performer',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_ra_replace_ta',
     'name': 'recording artist effect on track artist',
     'value': 'replace',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_ra_noblank_ta',
     'name': 'disallow blank recording artist',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_ra_merge_ta',
     'name': 'recording artist effect on track artist',
     'value': 'merge',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_composer_album',
     'name': 'Album prefix',
     # 'value': 'Composer', # Can't use 'value' if there is only one option, otherwise False will revert to default
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_arrangers',
     'name': 'include arrangers',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_no_lyricists',
     'name': 'exclude lyricists if no vocals',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_cyrillic',
     'name': 'fix cyrillic',
     'type': 'Boolean',
     'default': True
     },
    # {'option': 'cea_genres',
    #  'name': 'infer work types',
    #  'type': 'Boolean',
    #  'default': True
    #  },
    # Note that the above is no longer used - replaced by cwp_genres_infer from v0.9.2
    {'option': 'cea_credited',
     'name': 'use release credited-as name',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_release_relationship_credited',
     'name': 'use release relationship credited-as name',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_group_credited',
     'name': 'use release-group credited-as name',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_recording_credited',
     'name': 'use recording credited-as name',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_recording_relationship_credited',
     'name': 'use recording relationship credited-as name',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_track_credited',
     'name': 'use track credited-as name',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_performer_credited',
     'name': 'use credited-as name for performer',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_composer_credited',
     'name': 'use credited-as name for composer',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_inst_credit',
     'name': 'use credited instrument',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_no_solo',
     'name': 'exclude solo',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_chorusmaster',
     'name': 'chorusmaster',
     'type': 'Text',
     'default': 'choirmaster'
     },
    {'option': 'cea_orchestrator',
     'name': 'orchestrator',
     'type': 'Text',
     'default': 'orch.'
     },
    {'option': 'cea_concertmaster',
     'name': 'concertmaster',
     'type': 'Text',
     'default': 'leader'
     },
    {'option': 'cea_lyricist',
     'name': 'lyricist',
     'type': 'Text',
     'default': 'lyrics'
     },
    {'option': 'cea_librettist',
     'name': 'librettist',
     'type': 'Text',
     'default': 'libretto'
     },
    {'option': 'cea_writer',
     'name': 'writer',
     'type': 'Text',
     'default': 'writer'
     },
    {'option': 'cea_arranger',
     'name': 'arranger',
     'type': 'Text',
     'default': 'arr.'
     },
    {'option': 'cea_reconstructed',
     'name': 'reconstructed by',
     'type': 'Text',
     'default': 'reconstructed'
     },
    {'option': 'cea_revised',
     'name': 'revised by',
     'type': 'Text',
     'default': 'revised'
     },
    {'option': 'cea_translator',
     'name': 'translator',
     'type': 'Text',
     'default': 'trans.'
     },
    {'option': 'cea_split_lyrics',
     'name': 'split lyrics',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cea_lyrics_tag',
     'name': 'lyrics',
     'type': 'Text',
     'default': 'lyrics'
     },
    {'option': 'cea_album_lyrics',
     'name': 'album lyrics',
     'type': 'Text',
     'default': 'albumnotes'
     },
    {'option': 'cea_track_lyrics',
     'name': 'track lyrics',
     'type': 'Text',
     'default': 'tracknotes'
     }
]

TAG_OPTIONS = [
    {'option': 'cea_blank_tag',
     'name': 'Tags to blank',
     'type': 'Text',
     'default': 'artist, artistsort'
     },
    {'option': 'cea_blank_tag_2',
     'name': 'Tags to blank 2',
     'type': 'Text',
     'default': 'performer:orchestra, performer:choir, performer:choir vocals'
     },
    {'option': 'cea_keep',
     'name': 'File tags to keep',
     'type': 'Text',
     'default': ''
     },
    {'option': 'cea_clear_tags',
     'name': 'Clear previous tags',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cea_tag_sort',
     'name': 'populate sort tags',
     'type': 'Boolean',
     'default': True
     }
]
#  (tag mapping detail lines)
default_list = [
    ('album_soloists, album_ensembles, album_conductors', 'artist, artists', False),
    ('recording_artists', 'artist, artists', True),
    ('soloist_names, ensemble_names, conductors', 'artist, artists', True),
    ('soloists', 'soloists, trackartist, involved people', False),
    ('release', 'release_name', False),
    ('ensemble_names', 'band', False),
    ('composers', 'artist', True),
    ('MB_artists', 'composer', True),
    ('arranger', 'composer', True)
]
TAG_DETAIL_OPTIONS = []
for i in range(0, 16):
    if i < len(default_list):
        default_source, default_tag, default_cond = default_list[i]
    else:
        default_source = ''
        default_tag = ''
        default_cond = False
    TAG_DETAIL_OPTIONS.append({'option': 'cea_source_' + str(i + 1),
                               'name': 'line ' + str(i + 1) + '_source',
                               'type': 'Combo',
                               'default': default_source
                               })
    TAG_DETAIL_OPTIONS.append({'option': 'cea_tag_' + str(i + 1),
                               'name': 'line ' + str(i + 1) + '_tag',
                               'type': 'Text',
                               'default': default_tag
                               })
    TAG_DETAIL_OPTIONS.append({'option': 'cea_cond_' + str(i + 1),
                               'name': 'line ' + str(i + 1) + '_conditional',
                               'type': 'Boolean',
                               'default': default_cond
                               })

WORKPARTS_OPTIONS = [
    {'option': 'classical_work_parts',
     'name': 'run work parts',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_collections',
     'name': 'include collection relations',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_allow_empty_parts',
     # allow parts to be blank if there is arrangement or partial text label
     # checked = split
     'name': 'allow-empty-parts',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_common_chars',
     # for use in strip_parent_from_work
     # where no match exists and substring elimination is used
     # this sets the minimum number of matching 'words' (words followed by punctuation/spaces or EOL)
     #  required before they will be eliminated
     # 0 => no elimination
     # default is 2 words
     'name': 'min common words to eliminate',
     'type': 'Integer',
     'default': 2
     },
    {'option': 'cwp_proximity',
     # proximity of new words in title comparison which will result in
     # infill words being included as well. 2 means 2-word 'gaps' of
     # existing words between new words will be treated as 'new'
     'name': 'in-string proximity trigger',
     'type': 'Integer',
     'default': 2
     },
    {'option': 'cwp_end_proximity',
     # proximity measure to be used when infilling to the end of the title
     'name': 'end-string proximity trigger',
     'type': 'Integer',
     'default': 1
     },
    {'option': 'cwp_split_hyphenated',
     # splitting of hyphenated words for matching purposes
     # checked = split
     'name': 'hyphen-splitting',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_substring_match',
     # Proportion of a string to be matched to a (usually larger) string for
     # it to be considered essentially similar
     'name': 'similarity threshold',
     'type': 'Integer',
     'default': 100
     },
    {'option': 'cwp_fill_part',
     # Fill part name with title text if it would otherwise
     # have no text other than arrangement or partial annotations
     'name': 'disallow empty part names',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_prepositions',
     'name': 'prepositions',
     'type': 'Text',
     'default': "a, the, in, on, at, of, after, and, de, d'un, d'une, la, le, no, from, &, e, ed, et, un,"
                " une, al, ala, alla"
     },
    {'option': 'cwp_removewords',
     'name': 'ignore prefixes',
     'type': 'Text',
     'default': ' part , act , scene, movement, movt, no. , no , n., n , nr., nr , book , the , a , la , le , un ,'
                ' une , el , il , tableau, from , KV ,Concerto in, Concerto'
     },
    {'option': 'cwp_synonyms',
     'name': 'synonyms',
     'type': 'PlainText',
     'default': '(1, one) / (2, two) / (3, three) / (&, and) / (Rezitativ, Recitativo, Recitative) / '
                '(Sinfonia, Sinfonie, Symphonie, Symphony) / (Arie, Aria) / '
                '(Minuetto, Menuetto, Minuetta, Menuet, Minuet) / (Bourée, Bouree , Bourrée)'
     },
    {'option': 'cwp_replacements',
     'name': 'replacements',
     'type': 'Text',
     'default': '(words to be replaced, replacement words) / (please blank me, ) / (etc, etc)'
     },
    {'option': 'cwp_titles',
     'name': 'Style',
     'value': 'Titles',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_works',
     'name': 'Style',
     'value': 'Works',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_extended',
     'name': 'Style',
     'value': 'Extended',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_hierarchical_works',
     'name': 'Work source',
     'value': 'Hierarchy',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_level0_works',
     'name': 'Work source',
     'value': 'Level_0',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_derive_works_from_title',
     'name': 'Derive works from title',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_movt_tag_inc',
     'name': 'movement tag inc num',
     'type': 'Text',
     'default': 'part, movement, subtitle'
     },
    {'option': 'cwp_movt_tag_exc',
     'name': 'movement tag exc num',
     'type': 'Text',
     'default': ''
     },
    {'option': 'cwp_movt_tag_inc1',
     'name': '1-level movement tag inc num',
     'type': 'Text',
     'default': 'movement'
     },
    {'option': 'cwp_movt_tag_exc1',
     'name': '1-level movement tag exc num',
     'type': 'Text',
     'default': ''
     },
    {'option': 'cwp_movt_no_tag',
     'name': 'movement num tag',
     'type': 'Text',
     'default': 'movementnumber'
     },
    {'option': 'cwp_movt_tot_tag',
     'name': 'movement tot tag',
     'type': 'Text',
     'default': 'movementtotal'
     },
    {'option': 'cwp_work_tag_multi',
     'name': 'multi-level work tag',
     'type': 'Text',
     'default': 'groupheading, work'
     },
    {'option': 'cwp_work_tag_single',
     'name': 'single level work tag',
     'type': 'Text',
     'default': ''
     },
    {'option': 'cwp_top_tag',
     'name': 'top level work tag',
     'type': 'Text',
     'default': 'top_work, style, grouping'
     },
    {'option': 'cwp_multi_work_sep',
     'name': 'multi-level work separator',
     'type': 'Combo',
     'default': ':'
     },
    {'option': 'cwp_single_work_sep',
     'name': 'single level work separator',
     'type': 'Combo',
     'default': ':'
     },
    {'option': 'cwp_movt_no_sep',
     'name': 'movement number separator',
     'type': 'Combo',
     'default': '.'
     },
    {'option': 'cwp_partial',
     'name': 'show partial recordings',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_partial_text',
     'name': 'partial text',
     'type': 'Text',
     'default': '(part)'
     },
    {'option': 'cwp_arrangements',
     'name': 'include arrangement of',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_arrangements_text',
     'name': 'arrangements text',
     'type': 'Text',
     'default': 'Arrangement:'
     },
    {'option': 'cwp_medley',
     'name': 'list medleys',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_medley_text',
     'name': 'medley text',
     'type': 'Text',
     'default': 'Medley'
     }
]
# Options on "Genres etc." tab

GENRE_OPTIONS = [
    {'option': 'cwp_genre_tag',
     'name': 'main genre tag',
     'type': 'Text',
     'default': 'genre'
     },
    {'option': 'cwp_subgenre_tag',
     'name': 'sub-genre tag',
     'type': 'Text',
     'default': 'sub-genre'
     },
    {'option': 'cwp_genres_use_file',
     'name': 'source genre from file',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_genres_use_folks',
     'name': 'source genre from folksonomy tags',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_genres_use_worktype',
     'name': 'source genre from work-type(s)',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_genres_infer',
     'name': 'infer genre from artist details(s)',
     'type': 'Boolean',
     'default': False
     },
    # Note that the "infer from artists" option was in  the "artists"
    # section - legacy from v0.9.1 & prior
    {'option': 'cwp_genres_filter',
     'name': 'apply filter to genres',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_genres_classical_main',
     'name': 'classical main genres',
     'type': 'PlainText',
     'default': 'Classical, Chamber music, Concerto, Symphony, Opera, Orchestral, Sonata, Choral, Aria, Ballet, '
                'Oratorio, Motet, Symphonic poem, Suite, Partita, Song-cycle, Overture, '
                'Mass, Cantata'
     },
    {'option': 'cwp_genres_classical_sub',
     'name': 'classical sub-genres',
     'type': 'PlainText',
     'default': 'Chant, Classical crossover, Minimalism, Avant-garde, Impressionist, Aria, Duet, Trio, Quartet'
     },
    {'option': 'cwp_genres_other_main',
     'name': 'general main genres',
     'type': 'PlainText',
     'default': 'Alternative music, Blues, Country, Dance, Easy listening, Electronic music, Folk, Folk / pop, '
                'Hip hop / rap, Indie,  Religious, Asian, Jazz, Latin, New age, Pop, R&B / Soul, Reggae, Rock, '
                'World music, Celtic folk, French Medieval'
     },
    {'option': 'cwp_genres_other_sub',
     'name': 'general sub-genres',
     'type': 'PlainText',
     'default': 'Song, Vocal, Christmas, Instrumental'
     },
    {'option': 'cwp_genres_arranger_as_composer',
     'name': 'treat arranger as for composer for genre-setting',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_genres_classical_all',
     'name': 'make tracks classical',
     'value': 'all',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_genres_classical_selective',
     'name': 'make tracks classical',
     'value': 'selective',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_genres_classical_exclude',
     'name': 'exclude "classical" from main genre tag',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_genres_flag_text',
     'name': 'classical flag',
     'type': 'Text',
     'default': '1'
     },
    {'option': 'cwp_genres_flag_tag',
     'name': 'classical flag tag',
     'type': 'Text',
     'default': 'is_classical'
     },
    {'option': 'cwp_genres_default',
     'name': 'default genre',
     'type': 'Text',
     'default': 'Other'
     },
    {'option': 'cwp_instruments_tag',
     'name': 'instruments tag',
     'type': 'Text',
     'default': 'instrument'
     },
    {'option': 'cwp_instruments_MB_names',
     'name': 'use MB instrument names',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_instruments_credited_names',
     'name': 'use credited instrument names',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_key_tag',
     'name': 'key tag',
     'type': 'Text',
     'default': 'key'
     },
    {'option': 'cwp_key_contingent_include',
     'name': 'contingent include key in workname',
     'value': 'contingent',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_key_never_include',
     'name': 'never include key in workname',
     'value': 'never',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_key_include',
     'name': 'include key in workname',
     'value': 'always',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_workdate_tag',
     'name': 'workdate tag',
     'type': 'Text',
     'default': 'work_year'
     },
    {'option': 'cwp_workdate_source_composed',
     'name': 'use composed for workdate',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_workdate_source_published',
     'name': 'use published for workdate',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_workdate_source_premiered',
     'name': 'use premiered for workdate',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_workdate_use_first',
     'name': 'use workdate sources sequentially',
     'value': 'sequence',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_workdate_use_all',
     'name': 'use all workdate sources',
     'value': 'all',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_workdate_annotate',
     'name': 'annotate dates',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_workdate_include',
     'name': 'include workdate in workname',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_period_tag',
     'name': 'period tag',
     'type': 'Text',
     'default': 'period'
     },
    {'option': 'cwp_periods_arranger_as_composer',
     'name': 'treat arranger as for composer for period-setting',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_period_map',
     'name': 'period map',
     'type': 'PlainText',
     'default': 'Early, -3000,800; Medieval, 800,1400; Renaissance, 1400, 1600; Baroque, 1600,1750; '
                'Classical, 1750,1820; Early Romantic, 1800,1850; Late Romantic, 1850,1910; '
                '20th Century, 1910,1975; Contemporary, 1975,2525'
     }
]
# Picard options which are also saved (NB only affects plugin processing - not main Picard processing)
PICARD_OPTIONS = [
    {'option': 'standardize_artists',
     'name': 'standardize artists',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'translate_artist_names',
     'name': 'translate artist names',
     'type': 'Boolean',
     'default': True
     },
]

# other options (not saved in file tags)
OTHER_OPTIONS = [
    {'option': 'use_cache',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_aliases',
     'name': 'replace with alias?',
     'value': 'replace',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_no_aliases',
     'name': 'replace with alias?',
     'value': 'no replace',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_aliases_all',
     'name': 'alias replacement type',
     'value': 'all',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_aliases_greek',
     'name': 'alias replacement type',
     'value': 'non-latin',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_aliases_tagged',
     'name': 'alias replacement type',
     'value': 'tagged works',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_aliases_tag_text',
     'name': 'use_alias tag text',
     'type': 'Text',
     'default': 'use_alias'
     },
    {'option': 'cwp_aliases_tags_all',
     'name': 'use_alias tags all',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'cwp_aliases_tags_user',
     'name': 'use_alias tags user',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_use_sk',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_write_sk',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_retries',
     'type': 'Integer',
     'default': 6
     },
    {'option': 'cwp_use_muso_refdb',
     'name': 'use Muso ref database',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_muso_genres',
     'name': 'use Muso classical genres',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_muso_classical',
     'name': 'use Muso classical composers',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_muso_dates',
     'name': 'use Muso composer dates',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_muso_periods',
     'name': 'use Muso periods',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_muso_path',
     'name': 'path to Muso database',
     'type': 'Text',
     'default': 'C:\\Users\\Public\\Music\\muso\\database'
     },
    {'option': 'cwp_muso_refdb',
     'name': 'name of Muso reference database',
     'type': 'Text',
     'default': 'Reference.xml'
     },
    {'option': 'log_error',
     'type': 'Boolean',
             'default': True
     },
    {'option': 'log_warning',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'log_debug',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'log_basic',
     'type': 'Boolean',
     'default': True
     },
    {'option': 'log_info',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'ce_version_tag',
     'type': 'Text',
     'default': 'stamp'
     },
    {'option': 'cea_options_tag',
     'type': 'Text',
     'default': 'comment'
     },
    {'option': 'cwp_options_tag',
     'type': 'Text',
     'default': 'comment'
     },
    {'option': 'cea_override',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'ce_tagmap_override',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'cwp_override',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'ce_genres_override',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'ce_options_overwrite',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'ce_no_run',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'ce_show_ui_tags',
     'type': 'Boolean',
     'default': False
     },
    {'option': 'ce_ui_tags',
     ## Note that this is not just for work parts (although that is the main use),
     # but cwp prefix is to make use of code for synonyms
     'name': 'tags for ui columns',
     'type': 'PlainText',
     'default': 'Work diff: (groupheading_DIFF, work_DIFF, top_work_DIFF, grouping_DIFF) / Part diff: (part_DIFF, movement_DIFF) / Missing file metadata: 002_important_warning'
     }
]

ARTIST_TYPE_ORDER = {'vocal': 1,
                  'instrument': 1,
                  'performer': 0,
                  'performing orchestra': 2,
                  'concertmaster': 3,
                  'conductor': 4,
                  'chorus master': 5,
                  'composer': 6,
                  'writer': 7,
                  'reconstructed by': 8,
                  'instrument arranger': 9,
                  'vocal arranger': 9,
                  'arranger': 11,
                  'orchestrator': 12,
                  'revised by': 13,
                  'lyricist': 14,
                  'librettist': 15,
                  'translator': 16
                  }

CYRILLIC_UPPER = {
        u'А': u'A',
        u'Б': u'B',
        u'В': u'V',
        u'Г': u'G',
        u'Д': u'D',
        u'Е': u'E',
        u'Ё': u'E',
        u'Ж': u'Zh',
        u'З': u'Z',
        u'И': u'I',
        u'Й': u'Y',
        u'К': u'K',
        u'Л': u'L',
        u'М': u'M',
        u'Н': u'N',
        u'О': u'O',
        u'П': u'P',
        u'Р': u'R',
        u'С': u'S',
        u'Т': u'T',
        u'У': u'U',
        u'Ф': u'F',
        u'Х': u'H',
        u'Ц': u'Ts',
        u'Ч': u'Ch',
        u'Ш': u'Sh',
        u'Щ': u'Sch',
        u'Ъ': u'',
        u'Ы': u'Y',
        u'Ь': u'',
        u'Э': u'E',
        u'Ю': u'Yu',
        u'Я': u'Ya'
    }
CYRILLIC_LOWER = {
    u'а': u'a',
    u'б': u'b',
    u'в': u'v',
    u'г': u'g',
    u'д': u'd',
    u'е': u'e',
    u'ё': u'e',
    u'ж': u'zh',
    u'з': u'z',
    u'и': u'i',
    u'й': u'y',
    u'к': u'k',
    u'л': u'l',
    u'м': u'm',
    u'н': u'n',
    u'о': u'o',
    u'п': u'p',
    u'р': u'r',
    u'с': u's',
    u'т': u't',
    u'у': u'u',
    u'ф': u'f',
    u'х': u'h',
    u'ц': u'ts',
    u'ч': u'ch',
    u'ш': u'sh',
    u'щ': u'sch',
    u'ъ': u'',
    u'ы': u'y',
    u'ь': u'',
    u'э': u'e',
    u'ю': u'yu',
    u'я': u'ya'
}

def tag_strings(pre):
    TAG_STRINGS = {
            'writer': (
                'composer',
                pre + '_writers',
                'composersort',
                pre + '_writers_sort'),
            'composer': (
                'composer',
                pre + '_composers',
                'composersort',
                pre + '_composers_sort'),
            'lyricist': (
                'lyricist',
                pre + '_lyricists',
                '~lyricists_sort',
                pre + '_lyricists_sort'),
            'librettist': (
                'lyricist',
                pre + '_librettists',
                '~lyricists_sort',
                pre + '_librettists_sort'),
            'revised by': (
                'arranger',
                pre + '_revisors',
                '~arranger_sort',
                pre + '_revisors_sort'),
            'translator': (
                'lyricist',
                pre + '_translators',
                '~lyricists_sort',
                pre + '_translators_sort'),
            'reconstructed by': (
                'arranger',
                pre + '_reconstructors',
                '~arranger_sort',
                pre + '_reconstructors_sort'),
            'arranger': (
                'arranger',
                pre + '_arrangers',
                '~arranger_sort',
                pre + '_arrangers_sort'),
            'instrument arranger': (
                'arranger',
                pre + '_arrangers',
                '~arranger_sort',
                pre + '_arrangers_sort'),
            'orchestrator': (
                'arranger',
                pre + '_orchestrators',
                '~arranger_sort',
                pre + '_orchestrators_sort'),
            'vocal arranger': (
                'arranger',
                pre + '_arrangers',
                '~arranger_sort',
                pre + '_arrangers_sort'),
            'performer': (
                'performer:',
                pre + '_performers',
                '~performer_sort',
                pre + '_performers_sort'),
            'instrument': (
                'performer:',
                pre + '_performers',
                '~performer_sort',
                pre + '_performers_sort'),
            'vocal': (
                'performer:',
                pre + '_performers',
                '~performer_sort',
                pre + '_performers_sort'),
            'performing orchestra': (
                'performer:orchestra',
                pre + '_ensembles',
                '~performer_sort',
                pre + '_ensembles_sort'),
            'conductor': (
                'conductor',
                pre + '_conductors',
                '~conductor_sort',
                pre + '_conductors_sort'),
            'chorus master': (
                'conductor',
                pre + '_chorusmasters',
                '~conductor_sort',
                pre + '_chorusmasters_sort'),
            'concertmaster': (
                'performer',
                pre + '~_leaders',
                '~performer_sort',
                pre + '_leaders_sort')}
    return TAG_STRINGS

INSERTIONS = ['writer',
              'lyricist',
              'librettist',
              'revised by',
              'translator',
              'arranger',
              'reconstructed by',
              'orchestrator',
              'instrument arranger',
              'vocal arranger',
              'chorus master']